/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// A new version of this control will be created soon and will be compatible with the new API.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    DellClarity.ChassisStatus = Class.extend({

        init: function (id, options) {
            this.defaultUrl = options.statesUrl;
            this.id = id;
            this.container = null;
        },

        render: function (container, dto) {

            this.container = container;

            //Creates a chassis empty
            $("#" + container).append("<div alt='' id='" + this.id + "' class='chassisFront'></div>");

            //Fills the chassis with the images
            this.loadStateImage(this.id);
        },

        loadStateImage: function (imageId) {

            var idRequest = new DellClarity.IdRequest();

            DellClarity.Ajax.PostJSON(this.defaultUrl, idRequest, this.whenDtoIsLoaded.bind(this));
        },

        whenDtoIsLoaded: function (response) {
            var states = response;
            var idContainer = this.id + "_map";

            var divGridSlots = "<div id='" + idContainer + "_slots' class='chassisGridSlot'></div>";
            $("#" + this.id).append(divGridSlots);

            var divGridLCD = "<div id='" + idContainer + "_lcd' class='chassisGridLCD'></div>";
            $("#" + this.id).append(divGridLCD);

            /*Positions of the slots in the chassis
            ---------------------------------
            | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 
            --------------------------------|
            | 9 | 10| 11| 12| 13| 14| 15| 16| 
            ---------------------------------
            |              17               |  
            ---------------------------------
            */

            //Number of the slots columns in the chassis
            var columnsOfSlots = 8;

            //Creates slots structure with divs
            for (var i = 1; i <= columnsOfSlots; i++) {

                var idSlotContainer = idContainer + "_" + i;
                var idSlotTop = idContainer + "_" + i + "_slot";
                var idSlotBottom = idContainer + "_" + (i + columnsOfSlots) + "_slot";

                //main div
                divSlot = "<div id='" + idSlotContainer + "' class='chassisColumnSlots''></div>";

                //divs for slots 1-8
                divSlotTop = "<div id='" + idSlotTop + "' " + "class='chassisSlot'" + "informational='' href=''></div>";

                //divs for slots 9-16
                divSlotBottom = "<div id='" + idSlotBottom + "' " + "class='chassisSlot'" + "informational='' href=''></div>";

                $("#" + idContainer + "_slots").append(divSlot);
                $("#" + idSlotContainer).append(divSlotTop);
                $("#" + idSlotContainer).append(divSlotBottom);
            };

            //div for LCD Module
            var idLCDModule = idContainer + "_17_slot";
            div = "<div id='" + idLCDModule + "' " + " informational='' href=''></div>";
            $("#" + idContainer + "_lcd").append(div);

            var idSlot;

            for (var item in states) {

                idSlot = "#" + idContainer + "_" + states[item].Slot + "_slot";
                idExtendSlot = "#" + idContainer + "_" + (states[item].Slot + columnsOfSlots) + "_slot";

                switch (states[item].State) {
                    case 1: //OK: Server is powered on and is operating normally

                        $(idSlot).attr("informational", "System: " + states[item].System + "|Model: " + states[item].Model + "<br/>Status: OK");

                        switch (states[item].Type) {
                            case 1: //Single-slot server
                                $(idSlot).addClass('chassisServerOK');
                                break;
                            case 2: //Multi-slot server
                                $(idSlot).addClass('chassisMultiSlotServerOK');
                                $(idExtendSlot).addClass('chassisSlotEmpty');
                                break;
                            case 3: //LCD Module
                                $(idSlot).addClass('chassisLCDOK');
                                break;
                            default:
                                break;
                        }
                        break;

                    case 2: //WARNING: Server is reporting a non-critical error.

                        $("#" + idContainer + "_" + states[item].Slot + "_slot").attr("informational", "System: " + states[item].System + "|Model: " + states[item].Model + "<br/>Status: Warning");

                        switch (states[item].Type) {
                            case 1: //Single-slot server
                                $(idSlot).addClass('chassisServerWarning');
                                break;
                            case 2: //Multi-slot server
                                $(idSlot).addClass('chassisMultiSlotServerWarning');
                                $(idExtendSlot).addClass('chassisSlotEmpty');
                                break;
                            case 3: //LCD Module
                                $(idSlot).addClass('chassisLCDWarning');
                                break;
                            default:
                                break;
                        }
                        break;

                    case 3: //ERROR: Server is reporting a critical error.

                        $("#" + idContainer + "_" + states[item].Slot + "_slot").attr("informational", "System: " + states[item].System + "|Model: " + states[item].Model + "<br/>Status: Error");

                        switch (states[item].Type) {
                            case 1: //Single-slot server
                                $(idSlot).addClass('chassisServerError');
                                break;
                            case 2: //Multi-slot server
                                $(idSlot).addClass('chassisMultiSlotServerError');
                                $(idExtendSlot).addClass('chassisSlotEmpty');
                                break;
                            case 3: //LCD Module
                                $(idSlot).addClass('chassisLCDError');
                                break;
                            default:
                                break;
                        }
                        break;

                    case 4: //OFF: Server is off.

                        $("#" + idContainer + "_" + states[item].Slot + "_slot").attr("informational", "System: " + states[item].System + "|Model: " + states[item].Model + "<br/>Status: Off");

                        switch (states[item].Type) {
                            case 1: //Single-slot server
                                $(idSlot).addClass('chassisServerOff');
                                break;
                            case 2: //Multi-slot server
                                $(idSlot).addClass('chassisMultiSlotServerOff');
                                $(idExtendSlot).addClass('chassisSlotEmpty');
                                break;
                            case 3: //LCD Module
                                $(idSlot).addClass('chassisLCDOff');
                                break;
                            default:
                                break;
                        }
                        break;

                    default:
                        break;
                }
            };
        }

    });
}
