/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// A new version of this control will be created soon and will be compatible with the new API.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    DellClarity.DateTimePicker = (function () {
        return {

            ShowDateTimePicker: function (datetimefield, options) {
                var modal = DellClarity.DateTimePicker.CreateDateTimePicker(datetimefield, options);
                modal.show();
                return modal;
            },

            CreateDateTimePicker: function (datetimefield, options) {
                var initialized = false;
                var defaultWidth = 500;

                return {
                    modalTemplateUrl: "templates/datetimepickertemplate.html",
                    modalTemplateName: "datetimepickerModalTemplate",

                    hourmode: ko.observable((options != null && options.hourmode != null) ? options.hourmode : '12'),

                    date: ko.observable(),

                    timeHH12: ko.observable("12"),
                    timeHH24: ko.observable("00"),
                    timeMM: ko.observable("00"),
                    timeSS: ko.observable("00"),
                    timeAMPM: ko.observable("AM"),

                    availableTimeHH12: ["01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12"],
                    availableTimeHH24: ["00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23"],
                    availableTimeMM: [
                        "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10",
                        "11", "12", "13", "14", "15", "16", "17", "18", "19", "20",
                        "21", "22", "23", "24", "25", "26", "27", "28", "29", "30",
                        "31", "32", "33", "34", "35", "36", "37", "38", "39", "40",
                        "41", "42", "43", "44", "45", "46", "47", "48", "49", "50",
                        "51", "52", "53", "54", "55", "56", "57", "58", "59"],
                    availableTimeSS: [
                        "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10",
                        "11", "12", "13", "14", "15", "16", "17", "18", "19", "20",
                        "21", "22", "23", "24", "25", "26", "27", "28", "29", "30",
                        "31", "32", "33", "34", "35", "36", "37", "38", "39", "40",
                        "41", "42", "43", "44", "45", "46", "47", "48", "49", "50",
                        "51", "52", "53", "54", "55", "56", "57", "58", "59"],
                    availableTimeAMPM: ["AM", "PM"],

                    accepted: ko.observable(false),

                    visible: ko.observable(false),

                    width: ko.observable(defaultWidth),

                    show: function (width) {
                        this.initialize();

                        if (width)
                            this.width(width);

                        this.visible(true);

                        $("#date").datepicker();
                    },

                    close: function () {
                        this.visible(false);

                        var element = $("#" + this.getModalId());

                        if (element.length != 0) {
                            ko.cleanNode(element.get(0));
                            element.remove();
                        }
                    },

                    accept: function () {

                        var datetime;

                        if (this.hourmode() == '12')
                            datetime = this.date() + ' ' + this.timeHH12() + ':' + this.timeMM() + ' ' + this.timeAMPM();
                        else
                            datetime = this.date() + ' ' + this.timeHH24() + ':' + this.timeMM();

                        var parsedDate = new Date(datetime);

                        if (isNaN(parsedDate)) {
                            alert('Please select a valid date.');
                            return;
                        }

                        //update sent in field
                        $(datetimefield).val(datetime).change();
                        $(datetimefield).focus();
                        //$(datetimefield).blur();

                        this.close();
                    },

                    getModalId: function () {
                        return "datetimepickerModal";
                    },

                    initialize: function () {
                        if (!initialized) {
                            $("body").bind("keyup.coreModal" + this.getModalId(), function (event) {
                                // 27 == "esc"
                                if (event.which == 27) {
                                    self.close();
                                    // unbind the event
                                    $("body").unbind("keyup.coreModal" + this.getModalId());
                                }
                            });
                            initialized = true;
                        }

                        DellClarity.Resources.loadTemplate(this.modalTemplateName, this.modalTemplateUrl);

                        var existingElement = $("#" + this.getModalId());

                        if (existingElement.length == 0) {
                            var element = $('<div id="' + this.getModalId() + '" data-bind="template: {name: modalTemplateName, data: $data }" />');
                            $('body').append(element);
                            ko.cleanNode(element.get(0));
                            ko.applyBindings(this, element.get(0));
                        }
                    }
                };
            }
        };
    })();
}
