/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// A new version of this control will be created soon and will be compatible with the new API.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    //Grid namespace
    DellClarity.Grid = {};

    //Grid Helpers
    DellClarity.Grid.Helpers = {
        setButtonState: function (element, enabled) {
            if (!enabled) {
                element.addClass('ui-state-disabled');
                element.attr('disabled', 'disabled');
            } else {
                element.removeClass('ui-state-disabled');
                element.removeAttr('disabled');
            }
        }
    };

    //Grid Column data types
    DellClarity.Grid.DataTypes = {
        String: 'string',
        Numeric: 'numeric',
        Date: 'date',
        DateTime: 'datetime',
        Status: 'status',
        Custom: 'custom'
    };

    //
    // Grid: shows data in tabular form.
    DellClarity.Grid.Control = Class.extend({
        defaultJqGridColumnOptions: {
            index: 'name',
            name: 'name',
            label: 'Name',
            width: 200,
            align: 'left',
            sortable: true,
            editable: true,
            search: true,
            hidden: false,
            dataType: 'string',  //CUSTOM
            stype: 'text',
            binder: function (value) { return value; },
            valueMap: null
        },

        defaultJqGridOptions: {
            ajaxGridOptions: {
                processData: false
            },
            allowFiltering: true, //CUSTOM
            allowSettings: true, //CUSTOM
            allowCache: false, //CUSTOM - turn off if you don't want the data of the response cached
            polling: false, //CUSTOM
            pollingInterval: 60000, //CUSTOM - default to 60 seconds
            borderWidth: 3, //CUSTOM - Indicates the width of the table borders. Overrides CSS value.
            altRows: true,
            datatype: 'json',
            emptyrecords: DellClarity.Localization.Translations.table.zeroRecords,
            shrinkToFit: true,
            autowidth: true,
            forceFit: true,
            rowNum: 20,
            multiselectWidth: 36,
            gridview: false, //TODO: check this parameter for DRAWER, it should be false.
            jsonReader: {
                repeatitems: false,
                id: 'id',
                root: function (result) {
                    return result.responseObj;
                },
                page: function (result) {
                    return result.criteriaObj.paginationObj.currentPage + 1;
                },
                records: function (result) {
                    return result.criteriaObj.paginationObj.totalItemsCount;
                },
                total: function (result) {
                    return Math.ceil(result.criteriaObj.paginationObj.totalItemsCount / result.criteriaObj.paginationObj.rowCountPerPage);
                }
            },
            loadtext: DellClarity.Localization.Translations.table.loadingData,
            nodatatext: DellClarity.Localization.Translations.table.zeroRecords,
            mtype: 'POST',
            multiselect: false,
            page: 1,
            pgbuttons: true,
            pginput: true,

            serializeGridData: function (postMessage) {

                var getSearchFilters = function (postMessage) {
                    var filters = new Object();

                    if (postMessage._search) {
                        for (var prop in postMessage) {
                            if (prop != '_search' &&
                                prop != 'nd' &&
                                    prop != 'rows' &&
                                        prop != 'page' &&
                                            prop != 'sidx' &&
                                                prop != 'sord')
                                filters[prop] = postMessage[prop];
                        }
                    }

                    return filters;
                };

                var request = new Object();
                var criteria = new Object();
                request.criteriaObj = criteria;

                //Paging
                var paging = new Object();
                paging.currentPage = postMessage.page - 1;
                paging.rowCountPerPage = postMessage.rows;
                criteria.paginationObj = paging;

                //Sorting
                var sort = new Object();
                sort.field = postMessage.sidx;
                sort.order = postMessage.sord == 'asc' ? DellClarity.DataSortingConstants.ASCENDING : DellClarity.DataSortingConstants.DESCENDING;
                criteria.sortObj = [sort];

                //Filters
                criteria.filterObj = [];
                var filters = getSearchFilters(postMessage);
                $.each(filters, function (filterName, filterValue) {

                    //if there is an array it is because there are 2 input boxes so we can render this as a range

                    if (filterValue.from != null || filterValue.to != null) {

                        if (!isNullOrEmpty(filterValue.from)) {
                            var filterDataFrom = new Object();
                            filterDataFrom.field = filterName;
                            filterDataFrom.op = '>';
                            var gmtFromValue = Date.parse(filterValue.from);
                            filterDataFrom.opTarget = [gmtFromValue];
                            criteria.filterObj.push(filterDataFrom);
                        }

                        if (!isNullOrEmpty(filterValue.to)) {
                            var filterDataTo = new Object();
                            filterDataTo.field = filterName;
                            filterDataTo.op = '<';
                            var gmtToValue = Date.parse(filterValue.to);
                            filterDataTo.opTarget = [gmtToValue];
                            criteria.filterObj.push(filterDataTo);
                        }
                    } else {
                        var filterData = new Object();
                        filterData.field = filterName;
                        filterData.op = '=';
                        filterData.opTarget = [filterValue];
                        criteria.filterObj.push(filterData);
                    }

                });

                return JSON.stringify(request);
            },
            toolbar: [false, 'both'],
            viewrecords: true,
            toppager: true,
            loadComplete: function (data) {

                var grid = $('#' + this.id);
                if (grid.getGridParam("reccount") == 0) {
                    if (grid.parent().find($('.ui-jqgrid-emptyGrid')).length == 0)
                        grid.parent().append("<div class='ui-jqgrid-emptyGrid'>" + grid.getGridParam("nodatatext") + "</div>");
                    grid.resetSelection();

                    var gridId = this.id;

                    $('#del_' + gridId + '_top').addClass("ui-state-disabled");
                    $('#edit_' + gridId + '_top').addClass("ui-state-disabled");
                    $('.enable-one').addClass("ui-state-disabled");
                    $('.enable-many').addClass("ui-state-disabled");
                    $('option.enable-one').attr('disabled', 'disabled');
                    $('option.enable-many').attr('disabled', 'disabled');
                } else {
                    $('.ui-jqgrid-emptyGrid').remove();
                }

                if (grid.getGridParam('allowCache') == true) {
                    grid.data("jqData", data.responseObj);
                }

                if (grid.getGridParam('polling') == true) {
                    var interval = parseInt(grid.getGridParam('pollingInterval'));
                    if (DellClarity.PollingToken) clearTimeout(DellClarity.PollingToken);
                    DellClarity.PollingToken = setTimeout(function () {
                        grid.trigger("reloadGrid", [{ current: true }]);
                    }, interval);
                }

                $(window).trigger('resizeGrid');
            }
        },
        refreshGrid: function () {
            var tableID = 'table1';

            if (this.options && this.options.tableId)
                tableID = this.options.tableId;

            var grid = $('#' + tableID);

            if (grid)
                grid.trigger("reloadGrid", [{ current: true }]);
        },
        init: function (containerId, options) {

            var mergedOptions = $.extend(true, $.extend(true, {}, this.defaultJqGridOptions), options);
            var jqGridOptions = this.mapOptions(mergedOptions);

            var container = $("#" + containerId);

            //Set pager
            var pagerSelector = '#pager';
            var existingPager = $("#" + containerId + " DIV.pager");
            if (existingPager.length == 1)
                pagerSelector = '#' + existingPager.get(0).id;
            else {
                var pagerId = containerId + "_pager";
                container.find('TABLE').after('<div id="' + pagerId + '" class="pager"></div>');
                pagerSelector = '#' + pagerId;
            }
            jqGridOptions.pager = pagerSelector;

            var grids = container
                .find("TABLE")
                .jqGrid(jqGridOptions);

            var taskOptions = {
                ownerId: containerId,
                pagerSelector: pagerSelector,
                tasks: options.tasks
            };
            grids.taskbar(taskOptions);

            $(window).resize(function () {
                grids.setGridWidth($('#' + containerId).width());
            });

            $(window).bind('resizeGrid', function () {
                try {
                    grids.setGridWidth($('#' + containerId).width());
                }
                catch (e) { }
            });
        },

        mapOptions: function (options) {

            options = this.mapColumns(options);

            return options;
        },

        mapColumns: function (options) {

            var self = this;
            var colModel = [];
            var defaultColumn = this.defaultJqGridColumnOptions;
            $.each(options.columns, function () {
                var mappedOption = $.extend(true, $.extend(true, {}, defaultColumn), this);

                //Set index for sorting and search
                mappedOption.index = mappedOption.name;

                //Filters
                if (mappedOption.hasOwnProperty('filterList')) {
                    mappedOption.stype = 'select';
                    mappedOption.searchoptions = {
                        dataUrl: DellClarity.Urls.getAppUrl(mappedOption.filterList),
                        buildSelect: function (items) {
                            return self.keyValueToHtmlSelect(items, true);
                        }
                    };
                }

                //Set options depending on the dataType
                switch (mappedOption.dataType) {
                    case DellClarity.Grid.DataTypes.Date:
                        {
                            mappedOption.align = 'right';
                            mappedOption.searchoptions = {
                                dataInit: function (el) {
                                    //TODO: add date picker
                                    $(el).datepicker(
                                    //    { dateFormat: 'MM/dd/yyyy' }
                                );
                                }
                            },
                            mappedOption.formatter = function (el, cval, opts) {
                                return mappedOption.binder(el);
                            };
                            break;
                        }
                    case DellClarity.Grid.DataTypes.DateTime:
                        {
                            mappedOption.align = 'right';
                            mappedOption.searchoptions = {
                                dataInit: function (el) {
                                    //								//TODO: add date picker
                                    //								$(el).datepicker(
                                    //								//    { dateFormat: 'MM/dd/yyyy' }
                                    //							);
                                }
                            },
                            mappedOption.formatter = function (el, cval, opts) {
                                return mappedOption.binder(el);
                            };
                            break;
                        }
                    case DellClarity.Grid.DataTypes.Numeric:
                        {
                            mappedOption.align = 'right';
                            break;
                        }
                    case DellClarity.Grid.DataTypes.Status:
                        {
                            var statusMap = mappedOption.valueMap;
                            if (statusMap) {
                                mappedOption.editable = false;
                                mappedOption.formatter = function (el, cval, opts) {
                                    var className = ($.first(statusMap, function (item, index) {
                                        return item.status == el;
                                    })).className;
                                    return '<div class="icon ' + className + '"></div>';
                                };
                            }
                            break;
                        }
                    case DellClarity.Grid.DataTypes.Custom:
                        {
                            mappedOption.formatter = function (el, cval, opts) {
                                return mappedOption.binder(el);
                            };
                            break;
                        }
                }

                colModel.push(mappedOption);
            });
            options.colModel = colModel;

            return options;
        },

        keyValueToHtmlSelect: function (list, blank, allowAll) {
            var template = new DellClarity.StringTemplate('<OPTION value="#{value}">#{text}</OPTION>');
            var html = ['<SELECT>'];
            if (blank)
                html.push(template.evaluate({ value: '', text: '' }));
            if (allowAll)
                html.push(template.evaluate({ value: 'All', text: '--All--' }));
            $.each(eval(list), function () {
                html.push(template.evaluate(this));
            });
            html.push('</SELECT>');

            return html.join('');
        }
    });


    /*************************************
    * jqGrid extensions
    *************************************/
    (function ($) {
        $.jgrid.extend({
            settingsView: function (p) {
                p = $.extend({
                    top: 0,
                    left: 0,
                    width: 200,
                    height: 'auto',
                    dataheight: 'auto',
                    modal: false,
                    drag: true,
                    beforeShowForm: null,
                    afterShowForm: null,
                    afterSubmitForm: null,
                    closeOnEscape: true,
                    ShrinkToFit: false,
                    jqModal: false,
                    saveicon: [true, "left", "ui-icon-disk"],
                    closeicon: [true, "left", "ui-icon-close"],
                    onClose: null,
                    colnameview: true,
                    closeAfterSubmit: true,
                    updateAfterCheck: false,
                    recreateForm: false
                }, $.jgrid.col, p || {});
                return this.each(function () {

                    var $t = this;
                    if (!$t.grid) {
                        return;
                    }
                    var onBeforeShow = typeof p.beforeShowForm === 'function' ? true : false;
                    var onAfterShow = typeof p.afterShowForm === 'function' ? true : false;
                    var onAfterSubmit = typeof p.afterSubmitForm === 'function' ? true : false;
                    var gID = $t.p.id,
                        dtbl = "ColTbl_" + gID,
                        IDs = { themodal: 'colmod' + gID, modalhead: 'colhd' + gID, modalcontent: 'colcnt' + gID, scrollelm: dtbl };
                    if (p.recreateForm === true && $("#" + IDs.themodal).html() != null) {
                        $("#" + IDs.themodal).remove();
                    }
                    if ($("#" + IDs.themodal).html() != null) {
                        if (onBeforeShow) {
                            p.beforeShowForm($("#" + dtbl));
                        }
                        $.jgrid.viewModal("#" + IDs.themodal, { gbox: "#gbox_" + gID, jqm: p.jqModal, jqM: false, modal: p.modal });
                        if (onAfterShow) {
                            p.afterShowForm($("#" + dtbl));
                        }
                    } else {
                        var dh = isNaN(p.dataheight) ? p.dataheight : p.dataheight + "px";

                        var formTemplate = new DellClarity.StringTemplate(
                            '<div id="#{dtbl}" class="settingsView" style="height: #{dh}">' +
                                '   <table>' +
                                    '       <thead><tr><th>Columns</th><th>View</th><th>Group by</th></tr></thead>' +
                                        '       <tbody>#{columns}</tbody>' +
                                            '   </table>' +
                                                '</div>');
                        var columnsHtml = [];
                        var columnTemplate = new DellClarity.StringTemplate('<tr>' +
                            '   <td>#{name}</td>' +
                            '   <td style="white-space: pre;"><input type="checkbox" id="#{idShow}" class="cbox" value="T" #{checkedShow} /></td>' +
                            '   <td style="white-space: pre;"><input type="checkbox" id="#{idGroup}" class="cbox" value="T" #{checkedGroup} /></td>' +
                            '</tr>');

                        $.each(this.p.colModel, function () {
                            if (!this.hidedlg) {
                                columnsHtml.push(columnTemplate.evaluate({
                                    idShow: "col_" + this.name + "_show",
                                    idGroup: "col_" + this.name + "_group",
                                    name: this.label,
                                    checkedShow: (this.hidden === false) ? 'checked="checked"' : '',
                                    checkedGroup: ''
                                }));
                            }
                        });

                        var formHtml = formTemplate.evaluate({
                            dtbl: dtbl,
                            dh: dh,
                            columns: columnsHtml.join('')
                        });

                        p.gbox = "#gbox_" + gID;

                        $.jgrid.createModal(IDs, formHtml, p, "#gview_" + $t.p.id, $("#gview_" + $t.p.id)[0]);

                        //Show/hide
                        $(":input[id$='_show']", "#" + dtbl).click(function (e) {
                            var cn = this.id.substr(4).replace("_show", "");
                            if (cn) {
                                if (this.checked) {
                                    $($t).jqGrid("showCol", cn);
                                } else {
                                    $($t).jqGrid("hideCol", cn);
                                }
                                if (p.ShrinkToFit === true) {
                                    $($t).jqGrid("setGridWidth", $t.grid.width - 0.001, true);
                                }
                            }
                            return this;
                        });

                        if (onBeforeShow) {
                            p.beforeShowForm($("#" + dtbl));
                        }
                        $.jgrid.viewModal("#" + IDs.themodal, { gbox: "#gbox_" + gID, jqm: p.jqModal, jqM: true, modal: p.modal });
                        if (onAfterShow) {
                            p.afterShowForm($("#" + dtbl));
                        }
                    }
                });
            }
        });
    })(jQuery);


    /**
    Controls for toolbar.
    **/
    (function ($) {
        $.jgrid.extend({
            navSeparatorAdd: function (elem, p) {
                p = $.extend({
                    sepclass: "ui-separator",
                    sepcontent: '',
                    position: "last"
                }, p || {});
                return this.each(function () {
                    if (!this.grid) {
                        return;
                    }
                    if (elem.indexOf("#") !== 0) {
                        elem = "#" + elem;
                    }
                    var findnav = $(".navtable", elem)[0];
                    if (findnav) {
                        var sep = "<td class='ui-pg-button' style='width:4px;'><span class='" + p.sepclass + "'></span>" + p.sepcontent + "</td>";
                        if (p.position == 'first') {
                            if (findnav.rows[0].cells.length === 0) {
                                $("tr", findnav).append(sep);
                            } else {
                                $("tr td:eq(0)", findnav).before(sep);
                            }
                        } else {
                            $("tr", findnav).append(sep);
                        }

                    }
                });
            },
            navTopRightButtonAdd: function (elem, p) {
                p = $.extend({
                    caption: "newButton",
                    title: '',
                    buttonicon: 'ui-icon-newwin',
                    onClickButton: null,
                    position: "last",
                    cursor: 'pointer'
                }, p || {});
                return this.each(function () {
                    if (!this.grid) {
                        return;
                    }
                    if (elem.indexOf("#") !== 0) {
                        elem = "#" + elem;
                    }
                    var findnav = $(".navRightTable", elem)[0], $t = this;
                    if (findnav) {
                        if (p.id && $("#" + p.id, findnav).html() !== null) {
                            return;
                        }
                        var tbd = $("<td></td>");
                        if (p.buttonicon.toString().toUpperCase() == "NONE") {
                            $(tbd).addClass('ui-pg-button ui-corner-all').append("<div class='ui-pg-div'>" + p.caption + "</div>");
                        } else {
                            $(tbd).addClass('ui-pg-button ui-corner-all').append("<div class='ui-pg-div'><span class='ui-icon " + p.buttonicon + "'></span>" + p.caption + "</div>");
                        }
                        if (p.id) {
                            $(tbd).attr("id", p.id);
                        }
                        if (p.position == 'first') {
                            if (findnav.rows[0].cells.length === 0) {
                                $("tr", findnav).append(tbd);
                            } else {
                                $("tr td:eq(0)", findnav).before(tbd);
                            }
                        } else {
                            $("tr", findnav).append(tbd);
                        }
                        $(tbd, findnav)
                            .attr("title", p.title || "")
                            .click(function (e) {
                                if (!$(this).hasClass('ui-state-disabled')) {
                                    if ($.isFunction(p.onClickButton)) {
                                        p.onClickButton.call($t, e);
                                    }
                                }
                                return false;
                            })
                            .hover(
                                function () {
                                    if (!$(this).hasClass('ui-state-disabled')) {
                                        $(this).addClass('ui-state-hover');
                                    }
                                },
                                function () { $(this).removeClass("ui-state-hover"); }
                            );
                    }
                });
            },
            navDropDownAdd: function (elem, p) {
                p = $.extend({
                    label: '',
                    href: 'javascript:void(0);',
                    position: 'last',
                    cursor: 'pointer',
                    styleClass: '',
                    enabledHandler: null,
                    handler: function () { }
                }, p || {});
                return this.each(function () {
                    if (!this.grid) {
                        return;
                    }
                    if (elem.indexOf("#") !== 0) {
                        elem = "#" + elem;
                    }
                    var findnav = $(".navtable", elem)[0], $t = this;
                    if (findnav) {
                        if (p.id && $("#" + p.id, findnav).html() !== null) {
                            return;
                        }
                        var tbd = $("<td></td>");
                        $(tbd).addClass('ui-link');

                        var linkID = "contextMenuLink" + p.id;

                        $(tbd).append("<a href='" + p.href + "' id='" + linkID + "' class='" + p.styleClass + "'>" + p.title + " <img src=\"dellclarity.core/images/icons/down-arrow-blue.png\" alt=\"\" /></a>");
                        if (p.id) {
                            $(tbd).attr("id", p.id);
                        }

                        if (p.position == 'first') {
                            if (findnav.rows[0].cells.length === 0) {
                                $("tr", findnav).append(tbd);
                            } else {
                                $("tr td:eq(0)", findnav).before(tbd);
                            }
                        } else {
                            $("tr", findnav).append(tbd);
                        }

                        // Attach enabled handler 
                        if ($.isFunction(p.enabledHandler)) {
                            $('#' + $t.id).on("setStateOfGridButtonsBySelection", function (e, isChecked) {

                                var selectedRowIds = $('#' + $t.id).jqGrid('getGridParam', 'selarrrow');
                                var selecteddata = [];

                                $.each(selectedRowIds, function (idx, rowId) {
                                    var rowData = $($t).jqGrid("getRowData", rowId);
                                    selecteddata.push(rowData);
                                });

                                p.enabledHandler(selecteddata, isChecked);
                            });
                        }

                        var options = {
                            position: function (opt, x, y) {
                                var offset;
                                var selectorOffset = $(opt.selector).offset();
                                offset = { left: selectorOffset.left, top: selectorOffset.top + $(opt.selector).height() };

                                if ($.browser.msie) {
                                    offset.top = offset.top - 5;
                                    offset.left = offset.left - 10;
                                }

                                opt.$menu.css(offset);
                            }
                        };

                        $('#' + linkID).quickMenu(p.contextItems, options);
                    }
                });
            },
            navLinkAdd: function (elem, p) {
                p = $.extend({
                    label: '',
                    href: 'javascript:void(0);',
                    position: 'last',
                    cursor: 'pointer',
                    styleClass: 'stdIconLink',
                    enabledHandler: null,
                    handler: function () { }
                }, p || {});
                return this.each(function () {
                    if (!this.grid) {
                        return;
                    }
                    if (elem.indexOf("#") !== 0) {
                        elem = "#" + elem;
                    }
                    var findnav = $(".navtable", elem)[0], $t = this;
                    if (findnav) {
                        if (p.id && $("#" + p.id, findnav).html() !== null) {
                            return;
                        }
                        var tbd = $("<td></td>");
                        $(tbd).addClass('ui-link');
                        $(tbd).append("<a href='" + p.href + "' class='" + p.styleClass + "'>" + p.label + "</a>");
                        if (p.id) {
                            $(tbd).attr("id", p.id);
                        }
                        $(tbd).click(
                            function () {
                                if (!$(tbd).hasClass('ui-state-disabled')) {
                                    p.handler();
                                }
                            }
                        );
                        if (p.position == 'first') {
                            if (findnav.rows[0].cells.length === 0) {
                                $("tr", findnav).append(tbd);
                            } else {
                                $("tr td:eq(0)", findnav).before(tbd);
                            }
                        } else {
                            $("tr", findnav).append(tbd);
                        }

                        // Attach enabled handler 
                        if ($.isFunction(p.enabledHandler)) {
                            $('#' + $t.id).on("setStateOfGridButtonsBySelection", function (e, isChecked) {

                                var selectedRowIds = $('#' + $t.id).jqGrid('getGridParam', 'selarrrow');
                                var selecteddata = [];

                                $.each(selectedRowIds, function (idx, rowId) {
                                    var rowData = $($t).jqGrid("getRowData", rowId);
                                    selecteddata.push(rowData);
                                });

                                p.enabledHandler(selecteddata, isChecked);
                            });
                        }

                    }
                });
            },
            navLabelAdd: function (elem, p) {
                p = $.extend({
                    label: '',
                    position: 'first',
                    cursor: 'default'
                }, p || {});
                return this.each(function () {
                    if (!this.grid) {
                        return;
                    }
                    if (elem.indexOf("#") !== 0) {
                        elem = "#" + elem;
                    }
                    var findnav = $(".navtable", elem)[0], $t = this;
                    if (findnav) {
                        if (p.id && $("#" + p.id, findnav).html() !== null) {
                            return;
                        }
                        var tbd = $("<td></td>");
                        $(tbd).addClass('ui-label');
                        $(tbd).append("<label style='cursor:default;'>" + p.label + "</label>");
                        if (p.id) {
                            $(tbd).attr("id", p.id);
                        }
                        if (p.position == 'first') {
                            if (findnav.rows[0].cells.length === 0) {
                                $("tr", findnav).append(tbd);
                            } else {
                                $("tr td:eq(0)", findnav).before(tbd);
                            }
                        } else {
                            $("tr", findnav).append(tbd);
                        }
                    }
                });
            },
            navSelectAdd: function (elem, p) {
                p = $.extend({
                    items: null,
                    position: 'last',
                    cursor: 'pointer',
                    blankItem: null
                }, p || {});
                return this.each(function () {
                    if (!this.grid) {
                        return;
                    }
                    if (elem.indexOf("#") !== 0) {
                        elem = "#" + elem;
                    }
                    var findnav = $(".navtable", elem)[0], $t = this;
                    if (findnav) {
                        if (p.id && $("#" + p.id, findnav).html() !== null) {
                            return;
                        }
                        var tbd = $("<td></td>");
                        $(tbd).addClass('ui-label');

                        var slt = $('<select id></select>');

                        if (p.blankItem)
                            $(slt).append($('<option></option>').html(p.blankItem));

                        $.each(p.items, function (index, item) {
                            $(slt).append($('<option></option>').val(item.id).html(item.name));
                        });
                        $(tbd).append(slt);
                        $(tbd).change(function (event) {
                            var selectedTask = $.first(p.items, function (item, index) {
                                return item.id == event.target.value;
                            });
                            if (selectedTask)
                                selectedTask.handler();

                            $('#' + p.id)[0].selectedIndex = 0;

                        });
                        if (p.id) {
                            $(slt).attr("id", p.id);
                        }
                        if (p.position == 'first') {
                            if (findnav.rows[0].cells.length === 0) {
                                $("tr", findnav).append(tbd);
                            } else {
                                $("tr td:eq(0)", findnav).before(tbd);
                            }
                        } else {
                            $("tr", findnav).append(tbd);
                        }

                        // Attach enabled handler 
                        $.each(p.items, function (i, menuOption) {

                            if ($.isFunction(menuOption.enabledHandler)) {
                                $('#' + $t.id).on("setStateOfGridButtonsBySelection", function (e, isChecked) {

                                    var selectedRowIds = $('#' + $t.id).jqGrid('getGridParam', 'selarrrow');
                                    var selecteddata = [];

                                    $.each(selectedRowIds, function (idx, rowId) {
                                        var rowData = $($t).jqGrid("getRowData", rowId);
                                        selecteddata.push(rowData);
                                    });

                                    menuOption.enabledHandler(selecteddata, isChecked);
                                });
                            }
                        });

                    }
                });
            }
        });
    })(jQuery);


    /**
    Taskbar
    **/
    DellClarity.Tasks = {};

    //
    // Represents the number of items in the context to which the task
    // can be applied.
    //
    DellClarity.Tasks.TaskAppliesTo = {
        NONE: 0,
        ONE: 1,
        MANY: 2,
        ALWAYS: 3
    };

    //
    // The importance of the task indicates how will it be rendered.
    // -Main: button
    // -Additional: dropdown list item
    // -External: link
    //
    DellClarity.Tasks.TaskImportance = {
        //Main task, represented as a button
        MAIN: 0,

        //Additional task, represented as an option in a dropdown list.
        ADDITIONAL: 1,

        //External task, does not apply to a specific item in the context and 
        //navigates away from the current page or opens a new window
        EXTERNAL: 2
    };


    (function ($) {
        $.jgrid.extend({
            taskbar: function (options) {

                if (!options.tasks || !options.tasks.items || options.tasks.items.length == 0)
                    return;

                var defaultTasksDefinition = [
                    {
                        id: 'Add',
                        name: 'Create',
                        appliesTo: DellClarity.Tasks.TaskAppliesTo.ALWAYS,
                        importance: DellClarity.Tasks.TaskImportance.MAIN,
                        handler: function () {
                        }
                    },
                    {
                        id: 'Delete',
                        name: 'Delete',
                        appliesTo: DellClarity.Tasks.TaskAppliesTo.MANY,
                        importance: DellClarity.Tasks.TaskImportance.MAIN,
                        handler: function () {
                        }
                    },
                    {
                        id: 'Edit',
                        name: 'Edit',
                        appliesTo: DellClarity.Tasks.TaskAppliesTo.ONE,
                        importance: DellClarity.Tasks.TaskImportance.MAIN,
                        handler: function () {
                        }
                    },
                    {
                        id: 'Default_MAIN',
                        name: '',
                        appliesTo: DellClarity.Tasks.TaskAppliesTo.ONE,
                        importance: DellClarity.Tasks.TaskImportance.MAIN,
                        handler: function () {
                        }
                    },
                    {
                        id: 'Default_ADDITIONAL',
                        name: '',
                        appliesTo: DellClarity.Tasks.TaskAppliesTo.ONE,
                        importance: DellClarity.Tasks.TaskImportance.ADDITIONAL,
                        handler: function () {
                        }
                    },
                    {
                        id: 'Default_EXTERNAL',
                        name: '',
                        appliesTo: DellClarity.Tasks.TaskAppliesTo.NONE,
                        importance: DellClarity.Tasks.TaskImportance.EXTERNAL,
                        handler: function () {
                        }
                    }
                ];


                //Merge tasks default settings
                var definedTasks = options.tasks;
                var defaultTasks = defaultTasksDefinition;
                for (var i = 0; i < definedTasks.items.length; i++) {
                    var definedTask = definedTasks.items[i];

                    //First search task by ID in the defaults
                    var defaultTask = $.first(defaultTasks, function (defaultTaskDefinition, index) {
                        return defaultTaskDefinition.id == definedTask.id;
                    });

                    //If not found by ID, search the default task definition by importance
                    if (!defaultTask) {
                        defaultTask = $.first(defaultTasks, function (defaultTaskDefinition, index) {
                            return defaultTaskDefinition.importance == definedTask.importance &&
                                defaultTaskDefinition.id.indexOf("Default_") == 0;
                        });
                    }

                    if (defaultTask)
                        definedTasks.items[i] = $.extend(true, $.extend(true, {}, defaultTask), definedTask);
                }

                options = $.extend({
                    title: options.tasks.title,
                    tasks: definedTasks,
                    hasTextMainTask: function (taskId) {
                        var task = $.first(this.tasks.items, function (task) {
                            if (task.id == taskId)
                                return task.name;
                        });
                        if (task)
                            return task.name;
                    },
                    hasFunctionMainTask: function (taskId) {
                        var task = $.first(this.tasks.items, function (task) {
                            if (task.id == taskId)
                                return task.handler;
                        });
                        if (task)
                            return task.handler;
                    },
                    hasMainTask: function (taskId) {
                        var task = $.first(this.tasks.items, function (task) {
                            return task.id == taskId;
                        });
                        return task != null;
                    },
                    hasMainTasks: function () {
                        return $.grep(this.tasks.items, function (item, index) {
                            return item.importance == DellClarity.Tasks.TaskImportance.MAIN;
                        }).length > 0;
                    },
                    hasExternalTasks: function () {
                        return $.grep(this.tasks.items, function (item, index) {
                            return item.importance == DellClarity.Tasks.TaskImportance.EXTERNAL;
                        }).length > 0;
                    },
                    hasAdditionalTasks: function () {
                        return $.grep(this.tasks.items, function (item, index) {
                            return item.importance == DellClarity.Tasks.TaskImportance.ADDITIONAL;
                        }).length > 0;
                    }
                }, options || {});

                return this.each(function () {

                    var filters = { exists: false, visible: false };

                    var topPagerSelector = $(options.pagerSelector).parent()[0].id.replace("gbox_", "") + "_toppager";

                    var topPagerOptions = {
                        edit: false,
                        add: false,
                        del: false,
                        search: false,
                        refresh: true
                    };

                    var self = this;

                    $(this).jqGrid('navGrid', topPagerSelector, topPagerOptions)
                    // removing grouping and column picker until it can be fully vetted by EDG
                    //					.jqGrid('navTopRightButtonAdd', topPagerSelector, {
                    //						caption: '',
                    //						title: 'Settings',
                    //						buttonicon: 'ui-icon-gear',
                    //						onClickButton: function(event) {
                    //							//TODO: find a better way to determine settings view position
                    //							$(self).settingsView({
                    //								top: 40,
                    //								left: $(self).width() - 250
                    //							});
                    //						}
                    //					})
                        .jqGrid('navTopRightButtonAdd', topPagerSelector, {
                            caption: '',
                            title: "Filters",
                            buttonicon: 'ui-icon-search',
                            onClickButton: function () {
                                if (!filters.exists) {
                                    $(self).jqGrid('filterToolbar');
                                    filters.exists = true;
                                    filters.visible = true;
                                    $('.ui-jqgrid-labels', $("#gview_" + this.id)).addClass('ui-jqgrid-labels-filter-active');
                                    $('.ui-jqgrid-hbox', $("#gview_" + this.id)).addClass('ui-jqgrid-hbox-filter-active');
                                    $('table.ui-pg-table .navRightTable td:last-child', $("#gview_" + this.id)).addClass('ui-jqgrid-btsearch-filter-active');
                                } else {
                                    self.toggleToolbar();
                                    if (filters.visible) {
                                        self.clearToolbar();
                                        $('.ui-jqgrid-labels', $("#gview_" + this.id)).removeClass('ui-jqgrid-labels-filter-active');
                                        $('.ui-jqgrid-hbox', $("#gview_" + this.id)).removeClass('ui-jqgrid-hbox-filter-active');
                                        $('table.ui-pg-table .navRightTable td:last-child', $("#gview_" + this.id)).removeClass('ui-jqgrid-btsearch-filter-active');
                                    } else {
                                        $('.ui-jqgrid-labels', $("#gview_" + this.id)).addClass('ui-jqgrid-labels-filter-active');
                                        $('.ui-jqgrid-hbox', $("#gview_" + this.id)).addClass('ui-jqgrid-hbox-filter-active');
                                        $('table.ui-pg-table .navRightTable td:last-child', $("#gview_" + this.id)).addClass('ui-jqgrid-btsearch-filter-active');
                                    }
                                    filters.visible = !filters.visible;

                                }
                            }
                        });

                    var hasExternalTasks = options.hasExternalTasks();
                    var hasAdditionalTasks = options.hasAdditionalTasks();
                    var hasMainTasks = options.hasMainTasks();

                    var hasAdditionalTasksSeparator = false;

                    var i = 0;
                    var item = null;
                    var lastIndex = options.tasks.items.length - 1;
                    for (i = lastIndex; i >= 0; i--) {
                        item = options.tasks.items[i];

                        // EXTERNAL
                        if (item.importance == DellClarity.Tasks.TaskImportance.EXTERNAL) {

                            $(self).jqGrid('navSeparatorAdd', topPagerSelector, {
                                position: 'first'
                            });

                            $(self).jqGrid('navLinkAdd', topPagerSelector, {
                                id: item.id + "_" + self.id + "_top",
                                label: item.name,
                                handler: item.handler,
                                position: 'first',
                                styleClass: item.styleClass,
                                enabledHandler: item.enabledHandler
                            });
                        }
                            // MAIN
                        else if (item.importance == DellClarity.Tasks.TaskImportance.MAIN) {

                            $(self).jqGrid('navSeparatorAdd', topPagerSelector, {
                                position: 'first'
                            });

                            $(self).jqGrid('navButtonAdd', topPagerSelector, {
                                id: item.id + "_" + self.id + "_top",
                                caption: item.name,
                                title: item.name,
                                buttonicon: item.styleClass != null ? item.styleClass : item.icon,
                                onClickButton: item.handler,
                                position: "first",
                                cursor: 'pointer',
                                enabledHandler: item.enabledHandler
                            });
                        }
                            // CONTEXT
                        else if (item.importance == DellClarity.Tasks.TaskImportance.CONTEXT) {
                            $(self).jqGrid('navSeparatorAdd', topPagerSelector, {
                                position: 'first'
                            });

                            $(self).jqGrid('navDropDownAdd', topPagerSelector, {
                                id: item.id + "_" + self.id + "_top",
                                caption: item.name,
                                title: item.name,
                                onClickButton: item.handler,
                                position: "first",
                                cursor: 'pointer',
                                contextItems: item.contextItems,
                                enabledHandler: item.enabledHandler
                            });
                        }
                            // ADDITIONAL
                        else if (item.importance == DellClarity.Tasks.TaskImportance.ADDITIONAL) {

                            if (!hasAdditionalTasksSeparator) {
                                $(self).jqGrid('navSeparatorAdd', topPagerSelector, {
                                    position: 'first'
                                });
                                hasAdditionalTasksSeparator = true;
                            }

                            $(self).jqGrid('navSelectAdd', topPagerSelector, {
                                id: options.ownerId + '_additionalTasks',
                                items: $.grep(options.tasks.items, function (item, index) {
                                    return item.importance == DellClarity.Tasks.TaskImportance.ADDITIONAL;
                                }),
                                blankItem: options.tasks.additionaltaskstitle != null ? options.tasks.additionaltaskstitle : DellClarity.Localization.Translations.table.tasks,
                                position: 'first'
                            });
                        }

                        switch (item.appliesTo) {
                            case DellClarity.Tasks.TaskAppliesTo.ALWAYS:
                                {
                                    $("#" + item.id + "_" + self.id + "_top").addClass('enable-always');
                                    break;
                                }
                            case DellClarity.Tasks.TaskAppliesTo.ONE:
                                {
                                    $("#" + item.id + "_" + self.id + "_top").addClass('ui-state-disabled enable-one');

                                    // disable option
                                    if (item.importance == DellClarity.Tasks.TaskImportance.ADDITIONAL) {
                                        $("#" + options.ownerId + '_additionalTasks option[value="' + item.id + '"]').addClass('ui-state-disabled enable-one');
                                        $("#" + options.ownerId + '_additionalTasks option[value="' + item.id + '"]').attr('disabled', 'disabled');
                                    }
                                    break;
                                }
                            case DellClarity.Tasks.TaskAppliesTo.MANY:
                                {
                                    $("#" + item.id + "_" + self.id + "_top").addClass('ui-state-disabled enable-many');

                                    // disable option
                                    if (item.importance == DellClarity.Tasks.TaskImportance.ADDITIONAL) {
                                        $("#" + options.ownerId + '_additionalTasks option[value="' + item.id + '"]').addClass('ui-state-disabled enable-many');
                                        $("#" + options.ownerId + '_additionalTasks option[value="' + item.id + '"]').attr('disabled', 'disabled');
                                    }
                                    break;
                                }
                        }
                    }

                    //				if (hasMainTasks || hasAdditionalTasks || hasExternalTasks) {
                    //					$(this).jqGrid('navLabelAdd', topPagerSelector, {
                    //						label: options.title + ':',
                    //						position: 'first'
                    //					});
                    //				}				

                });
            }
        });
    })(jQuery);


    /*******************************************************
    CustomGrid: Base grid for custom grid definitions.
    *******************************************************/

    DellClarity.Grid.Custom = DellClarity.Grid.Control.extend({
        init: function (instanceOptions) {

            this.mergeOptions(instanceOptions);

            new DellClarity.Grid.Control(this.options.id, this.options);
        },

        mergeOptions: function (instanceOptions) {

            this.options = $.extend(true, $.extend(true, {}, this.defaultOptions), instanceOptions);

            //Merge default column settings
            var mergedColumnModel = [];
            var defaultColumn = this.defaultOptions.defaultColumn;
            $.each(this.options.columns, function () {
                mergedColumnModel.push($.extend(true, $.extend(true, {}, defaultColumn), this));
            });
            this.options.columns = mergedColumnModel;

            if (this.options.sortorder != null && this.options.sortorder == DellClarity.DataSortingConstants.DESCENDING) {
                this.options.sortorder = 'desc';
            }
            else if (this.options.sortorder != null && this.options.sortorder == 'desc') {
                this.options.sortorder = 'desc';
            }
            else {
                this.options.sortorder = 'asc';
            }

            //        this.tableDefinition.pager.pageChanged = this.closeDetails;

            //        this.tableDefinition.details.open = (function (selectedItem) {
            //            this.loadDetailsView(selectedItem.id);
            //        }).bind(this);

            //        this.tableDefinition.details.close = (function () {
            //            this.closeDetails();
            //        }).bind(this);
        }
    });
}
