/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// A new version of this control will be created soon and will be compatible with the new API.
/// </summary>
if (DellClarity.Config.apiVersion <= 2.5) {
    DellClarity.ScrollableTable = (function () {

        var defaultOptions = {
            targetId: 'logsTable',
            height: 250,
            numberOfRows: 20,
            defaultSortOrder: DellClarity.Constants.DataSortingConstants.ASCENDING,
            enablePolling: false,
            enableSelect: true,
            multiselect: false,
            onRowSelect: null,
            onDataLoadComplete: null,
            onDataLoadBegin: null,
            onBindComplete: null,
            pollingInterval: 60000,
            boundingContainer: null,
            autoResize: true,
            useRestfulInterface: false,
            requestObj: null,
            autoSelectFirstRow: true,
            ajaxErrorHandler: function (error) {
                if (error) {
                    if (error.fldErrors) {
                        var errorCodeField = ko.utils.arrayFirst(error.fldErrors, function (errorProp) {
                            return errorProp.field == 'Error Code';
                        });
                        if (errorCodeField && (errorCodeField.errorMessage == 999 || errorCodeField.errorMessage == 0))
                            return;
                    }

                    throw error.errorMessage;
                }
                else
                    throw DellClarity.Localization.Translations.core.genericexception;
            },
            statusIndicator: 'none'  // 'modal'
        };

        var returnTable = {
            templateName: 'templates/tablepagertemplate.html',

            getBlankRequestObject: function () {
                var request = new Object();
                request.requestObj = new Object();

                var criteria = new Object();
                request.criteriaObj = criteria;

                //Paging
                var paging = new Object();
                paging.currentPage = null;
                paging.rowCountPerPage = null;
                criteria.paginationObj = paging;

                //Sorting
                criteria.sortObj = [];

                //Filters
                criteria.filterObj = [];

                return request;
            },

            getDefaultPaginationObject: function (currentPage, numberOfRows) {
                //Paging
                var paging = new Object();
                paging.currentPage = currentPage;
                paging.rowCountPerPage = numberOfRows;
                paging.totalItemsCount = 0;
                return paging;
            },

            create: function (opts) {

                var options = $.extend(true, $.extend(true, {}, defaultOptions), opts);

                var defaultSortObject = new Object();
                if (options.defaultSortField) {
                    defaultSortObject.field = options.defaultSortField;
                    defaultSortObject.order = options.defaultSortOrder && (options.defaultSortOrder == DellClarity.Constants.DataSortingConstants.DESCENDING || options.defaultSortOrder == DellClarity.Constants.DataSortingConstants.ASCENDING)
                        ? options.defaultSortOrder
                        : DellClarity.Constants.DataSortingConstants.ASCENDING;
                } else {
                    defaultSortObject = null;
                }

                var retVal = {
                    timeout: null,
                    tableInitialized: false,
                    targetId: ko.observable(options.targetId),
                    url: ko.observable(options.url),
                    height: ko.observable(options.height != null ? options.height : 300),
                    pollingInterval: ko.observable(options.pollingInterval != null ? options.pollingInterval : 60000),
                    data: ko.observableArray(),
                    sortObj: ko.observableArray([defaultSortObject]),
                    requestObj: ko.observable(options.requestObj),
                    paginationObj: ko.observable(DellClarity.ScrollableTable.getDefaultPaginationObject(0, options.numberOfRows)),
                    filterObj: ko.observableArray(),
                    selectedRowId: ko.observable(),
                    scrollbarPosition: ko.observable(),
                    multiselect: ko.observable(options.multiselect),
                    enableSelect: ko.observable(options.enableSelect),
                    boundingContainer: ko.observable(options.boundingContainer),
                    autoResize: ko.observable(options.autoResize),
                    loading: ko.observable(false),
                    useRestfulInterface: ko.observable(options.useRestfulInterface),
                    autoSelectFirstRow: ko.observable(options.autoSelectFirstRow),
                    statusIndicator: null,
                    selectedPages: [],
                    currentPageTotalRowCount: 0,
                    currentPageSelectionCount: 0,

                    onRowSelect: function (id) {
                        if (!options.enableSelect) return;

                        if (options.onRowSelect) {
                            options.onRowSelect(id);
                        }
                    },
                    onDataLoadComplete: function (data) {
                        this.hideWaitStatus();

                        if (options.onDataLoadComplete) {
                            options.onDataLoadComplete(data);
                        }
                    },
                    onDataLoadBegin: function () {
                        this.showWaitStatus();

                        if (options.onDataLoadBegin) {
                            options.onDataLoadBegin();
                        }
                    },
                    onBindComplete: function (data) {
                        if (!options.onBindComplete) return;

                        if (options.onBindComplete) {
                            options.onBindComplete(data);
                        }
                    },
                    hideWaitStatus: function () {
                        if (options.statusIndicator == 'modal') {
                            if (this.statusIndicator)
                                this.statusIndicator.close();
                        }
                    },
                    showWaitStatus: function () {
                        if (options.statusIndicator == 'modal') {
                            this.statusIndicator = DellClarity.Controls.StatusIndicatorModal.create().show();
                        }
                    },
                    refresh: function () {
                        var self = this;
                        self.loadData();
                    },
                    checkAll: function (checked) {
                        var self = this;

                        ko.utils.arrayForEach(self.data(), function (row) {
                            row.checked(checked);
                        });

                        self.currentPageSelectionCount = checked ? self.currentPageTotalRowCount : 0;

                        return true;
                    },
                    uncheckAll: function () {
                        var self = this;

                        ko.utils.arrayForEach(self.data(), function (row) {
                            row.checked(false);
                        });

                        return true;
                    },
                    selectRow: function (data) {
                        var self = retVal;
                        if (!self.enableSelect()) return;
                        self.selectedRowId(data.id);
                        self.onRowSelect(data.id);
                    },
                    updateSort: function (field) {
                        var self = this;

                        var currentSortObj = self.sortObj()[0];

                        if (currentSortObj.field == field) {
                            if (currentSortObj.order == DellClarity.Constants.DataSortingConstants.DESCENDING)
                                currentSortObj.order = DellClarity.Constants.DataSortingConstants.ASCENDING;
                            else currentSortObj.order = DellClarity.Constants.DataSortingConstants.DESCENDING;
                        } else {
                            currentSortObj.field = field;
                            currentSortObj.order = DellClarity.Constants.DataSortingConstants.ASCENDING;
                        }

                        self.refresh();
                    },
                    nextPage: function () {
                        var self = this;
                        var page = self.currentPage() + 1;
                        if (page > 0 && page <= self.lastPage()) {
                            self.currentPage(page);
                        }
                    },
                    previousPage: function () {
                        var self = this;
                        var page = self.currentPage() - 1;
                        if (page > 0 && page <= self.lastPage()) {
                            self.currentPage(page);
                        }
                    },
                    updatePage: function (page) {
                        var self = this;
                        if (page > 0 && page <= self.lastPage()) {
                            self.currentPage(page);
                        }
                    },
                    rowSelected: function (data) {
                        //if (data.checked() == true) return 'selected';
                        if (!isNullUndefinedOrEmpty(retVal.selectedRowId()) && data.id == retVal.selectedRowId()) return 'selected';
                        return '';
                    },
                    addFilter: function (viewModelProperty, elementSelector, operator) {
                        var self = this;

                        var element = $(elementSelector);

                        if (element && element.length > 0 && !isNullOrEmpty(element.val())) {
                            var filterData = new Object();
                            filterData.field = viewModelProperty;
                            filterData.op = operator ? operator : '=';
                            filterData.opTarget = [element.val()];
                            self.filterObj.push(filterData);
                        }

                    },
                    loadData: function () {

                        var self = this;

                        //Check if a load operation is in progress.
                        if (self.loading() === true)
                            return;

                        if ($("#" + self.targetId()).length == 0) return;

                        self.loading(true);
                        self.onDataLoadBegin();

                        var request = DellClarity.ScrollableTable.getBlankRequestObject();

                        if (self.requestObj())
                            request.requestObj = self.requestObj();
                        
                        request.criteriaObj.paginationObj = self.paginationObj();
                        request.criteriaObj.sortObj = self.sortObj();
                        request.criteriaObj.filterObj = self.filterObj();

                        var successHandler = function (response, data) {

                            if (data.criteriaObj) {
                                self.paginationObj(data.criteriaObj.paginationObj);
                                //self.sortObj(data.criteriaObj.sortObj);
                                //self.filterObj(data.criteriaObj.filterObj);

                                self.sortObj.valueHasMutated();
                                self.filterObj.valueHasMutated();
                            }

                            self.updateSelections(response);

                            self.onDataLoadComplete(response);

                            self.data(valueOrDefault(response, []));

                            self.onBindComplete(response);

                            self.renderTable();
                            self.loading(false);

                            if (options.enablePolling) {
                                clearTimeout(self.timeout);
                                self.timeout = setTimeout(function () {
                                    self.loadData();
                                }, options.pollingInterval);
                            }

                            if (self.enableSelect()) {
                                var table = $("#" + self.targetId());
                                table.find('td input[type="checkbox"]').shiftSelectableKO();
                            }
                        };

                        var errorHandler = function (error) {
                            self.hideWaitStatus();

                            if (options.ajaxErrorHandler)
                                options.ajaxErrorHandler(error);
                        };

                        if (self.useRestfulInterface())
                            self.ajax = DellClarity.Ajax.GetRestfulJSON(self.url(), null, request, successHandler, errorHandler);
                        else
                            self.ajax = DellClarity.Ajax.PostJSON(self.url(), request, successHandler, errorHandler);

                    },
                    clearTimeout: function () {
                        var self = this;
                        clearTimeout(self.timeout);
                    },
                    pausePolling: function () {
                        var self = this;
                        if (options.enablePolling) {
                            clearTimeout(self.timeout);
                        }
                    },
                    refreshTable: function () {
                        var self = this;
                        self.renderTable();
                    },
                    renderTable: function () {
                        var self = this;

                        var table = $("#" + self.targetId());

                        //save off scrollbar position before redrawing table
                        var currentTable = table.closest('.fht-tbody');
                        self.scrollbarPosition(currentTable.scrollTop());

                        table.fixedHeaderTable('destroy');

                        self.tableInitialized = false;

                        var tableContainer = null;
                        var needsScrollbar = false;

                        if (options.boundingContainer && $(options.boundingContainer).length > 0) {
                            tableContainer = $(options.boundingContainer);
                        } else if ($('#masterContainer').length > 0 || $('.masterContainer').length > 0) {
                            tableContainer = $('#masterContainer,.masterContainer');
                        } else if ($('#contentScrollBar').length > 0) {
                            tableContainer = $('#contentScrollBar');
                        }

                        //update height of table based on available height
                        if (tableContainer && tableContainer.length > 0) {
                            if (table.height() > tableContainer.height())
                                needsScrollbar = true;
                        }

                        //table is empty so don't bother adding vertical scrollbar
                        if (self.data().length > 0 && needsScrollbar) {

                            table.fixedHeaderTable(
                                {
                                    footer: true,
                                    cloneHeadToFoot: false,
                                    autoShow: true,
                                    height: self.height(),
                                    autoResize: false,
                                    create: function () {

                                        var currentHeight = table.height() - 25;

                                        var scrollablewrapper = table.closest('.fht-tbody');

                                        if (currentHeight < self.height() - 75) {
                                            scrollablewrapper.height(currentHeight);
                                        }

                                        if (!self.tableInitialized) {
                                            var element = $(table).closest('.fht-table-wrapper').find('.fht-thead thead');
                                            ko.cleanNode(element[0]);
                                            ko.applyBindings(self, element[0]);
                                        }

                                        //restore scroll position
                                        if (self.scrollbarPosition()) {
                                            scrollablewrapper.scrollTop(self.scrollbarPosition());
                                        }

                                        self.tableInitialized = true;
                                    }
                                });

                        }

                        //Select All
                        if (self.enableSelect()) {
                            self.markCheckAll(table);
                        }
                    },


                    markCheckAll: function (table) {
                        var self = this;

                        //1) Mark the checkbox default state
                        //var totalRows = $("#" + self.targetId() + " tbody tr").length;
                        //var checkedRows = $("#" + self.targetId() + " tbody tr td input:checked").length;
                        //var mustCheck = totalRows == checkedRows;

                        table.off('click', 'th.checkboxColumn input[type="checkbox"]');

                        var checkbox = $('th.checkboxColumn input[type="checkbox"]');
                        var mustCheck = self.currentPageTotalRowCount == self.currentPageSelectionCount;
                        if (self.currentPageTotalRowCount == 0)
                            mustCheck = false;

                        checkbox.prop('checked', mustCheck);

                        table.on('click', 'th.checkboxColumn input[type="checkbox"]', function () {
                            self.checkAll($(this).prop('checked'));
                        });

                        setTimeout(function () {
                            table.off('click', 'td.checkboxColumn input[type="checkbox"]');
                            table.on('click', 'td.checkboxColumn input[type="checkbox"]', function () {

                                var currentCheckboxStatus = $(this).prop('checked');
                                if (currentCheckboxStatus === true)
                                    self.currentPageSelectionCount++;
                                else
                                    self.currentPageSelectionCount--;

                                //var tdTotalRows = $("#" + self.targetId() + " tbody tr").length;
                                //var tdCheckedRows = $("#" + self.targetId() + " tbody tr td input:checked").length;
                                //var tdMustCheck = tdTotalRows == tdCheckedRows;
                                checkbox.prop('checked', self.currentPageTotalRowCount == self.currentPageSelectionCount);
                            });
                        });
                    },

                    updateSelections: function (response) {
                        var self = this;

                        var selectedRowId = self.selectedRowId();
                        var selectedRowIdExists = false;

                        if (self.enableSelect()) {
                            var table = $("#" + self.targetId());
                            var selectedItemsCount = 0;

                            ko.utils.arrayForEach(response, function (row) {

                                var selectedItem = ko.utils.arrayFirst(self.getSelections(), function (item) {
                                    var rowId = typeof item.id == 'function' ? item.id() : item.id;
                                    return rowId == row.id;
                                });
                                if (selectedItem) {
                                    row.checked = ko.observable(true);
                                    selectedItemsCount++;
                                } else {
                                    row.checked = ko.observable(false);
                                }

                                // if this is our selected row update the observable
                                if (selectedRowId && selectedRowId == row.id) {
                                    self.selectedRowId(row.id);
                                    selectedRowIdExists = true;
                                }

                            });

                            //This two variables (currentPageTotalRowCount and currentPageSelectionCount) represent cached calculations for each page refresh, that are used
                            //when each row selection is done. The reason for this is that calculating all this on each
                            //selection makes the UI unresponsive.
                            self.currentPageTotalRowCount = response.length;
                            self.currentPageSelectionCount = selectedItemsCount;

                            if (!selectedRowIdExists) {
                                if (response.length > 0 && self.autoSelectFirstRow()) {
                                    self.selectedRowId(response[0].id);
                                    self.onRowSelect(response[0].id);
                                } else {
                                    self.selectedRowId(null);
                                }
                            } else {
                                self.onRowSelect(self.selectedRowId());
                            }
                        }
                    }
                };

                retVal.getSelections = function () {

                    if (retVal.enableSelect()) {
                        var currentPage = retVal.currentPage();

                        var selectedPage = ko.utils.arrayFirst(retVal.selectedPages, function (item) {
                            return item.page == currentPage;
                        });
                        if (selectedPage)
                            return selectedPage.selectedItems;
                    }

                    return [];
                };

                retVal.saveSelections = function () {
                    var self = this;

                    if (self.enableSelect()) {
                        var currentPage = self.currentPage();

                        var selectedPage = ko.utils.arrayFirst(self.selectedPages, function (item) {
                            return item.page == currentPage;
                        });

                        var checkedRows = [];
                        $.each(self.data(), function (i, row) {
                            if (row.checked())
                                checkedRows.push(row);
                        });
                        if (selectedPage)
                            selectedPage.selectedItems = checkedRows;
                        else
                            self.selectedPages.push({ page: currentPage, selectedItems: checkedRows });
                    }
                };

                /* Pagination Helpers */
                retVal.itemCount = ko.computed(function () {
                    return retVal.paginationObj().totalItemsCount.format('n0');
                });

                retVal.currentPage = ko.computed({
                    read: function () {
                        return retVal.paginationObj().currentPage + 1;
                    },
                    write: function (value) {
                        var page = value - 1;
                        if (page >= 0 && page < retVal.lastPage()) {
                            {
                                retVal.saveSelections();
                                retVal.paginationObj().currentPage = page;
                                retVal.refresh();
                            }
                        } else {
                            retVal.paginationObj.valueHasMutated();
                        }
                    }
                });

                retVal.lastPage = ko.computed(function () {
                    return Math.ceil(retVal.paginationObj().totalItemsCount / retVal.paginationObj().rowCountPerPage);
                });

                retVal.firstDisplayedRowNumber = ko.computed(function () {
                    if (!retVal.data || !retVal.data() || retVal.data().length == 0) return 0;
                    return ((retVal.paginationObj().currentPage * retVal.paginationObj().rowCountPerPage) + 1).format('n0');
                });

                retVal.lastDisplayedRowNumber = ko.computed(function () {
                    return ((retVal.paginationObj().currentPage * retVal.paginationObj().rowCountPerPage) + retVal.data().length).format('n0');
                });

                retVal.previousPageEnabled = ko.computed(function () {
                    return retVal.paginationObj().currentPage + 1 > 1;
                });

                retVal.nextPageEnabled = ko.computed(function () {
                    return retVal.paginationObj().currentPage + 1 < retVal.lastPage();
                });

                /* Helpers */

                retVal.selectedItemCount = ko.computed(function () {
                    var count = 0;
                    if (retVal.multiselect()) {

                        $.each(retVal.data(), function (i, row) {
                            if (row.checked()) count++;
                        });

                    } else {

                        if (!isNullOrEmpty(retVal.selectedRowId())) {
                            count = 1;
                        }

                    }
                    return count;
                });

                //Obsolete. Use selectedRowData observable instead. Left for compatibility.
                retVal.checkedRows = function () {
                    return ko.utils.arrayMap(retVal.selectedRowData(), function (item) {
                        return (typeof item.id == 'function' ? item.id() : item.id);
                    });
                };

                retVal.selectedRowData = ko.computed(function () {
                    var data = [];
                    if (retVal.multiselect()) {
                        $.each(retVal.selectedPages, function (i, selectedPage) {
                            if (retVal.currentPage() != selectedPage.page) {
                                $.each(selectedPage.selectedItems, function (j, row) {
                                    data.push(row);
                                });
                            }
                        });
                        $.each(retVal.data(), function (j, row) {
                            if (row.checked() == true)
                                data.push(row);
                        });

                    } else {

                        if (!isNullOrEmpty(retVal.selectedRowId())) {
                            $.each(retVal.data(), function (j, row) {
                                if (row.id == retVal.selectedRowId())
                                    data.push(row);
                            });
                        }
                    }
                    return data;
                });

                retVal.anyItemSelected = ko.computed(function () {
                    return retVal.selectedItemCount() > 0;
                });

                retVal.singleItemSelected = ko.computed(function () {
                    return retVal.selectedItemCount() == 1;
                });

                retVal.manyItemsSelected = ko.computed(function () {
                    return retVal.selectedItemCount() > 1;
                });

                $(window).bind('resizeTable', function () {
                    try {

                        if (options.autoResize) {
                            var container = null;

                            if (options.boundingContainer && $(options.boundingContainer).length > 0) {
                                container = $(options.boundingContainer);
                            } else if ($('#masterContainer').length > 0 || $('.masterContainer').length > 0) {
                                container = $('#masterContainer,.masterContainer');
                            } else if ($('#contentScrollBar').length > 0) {
                                container = $('#contentScrollBar');
                            }

                            //update height of table based on available height
                            if (container && container.length > 0) {
                                container.each(function (index, parentContainer) {
                                    var containerHeight = $(parentContainer).height();
                                    var tableElement = $(parentContainer).find(".fht-table-wrapper");
                                    if (tableElement && tableElement.length > 0) {
                                        var topOfTable = tableElement.position().top;
                                        retVal.height(containerHeight - topOfTable);
                                    }
                                });
                            }
                        }

                        retVal.refreshTable();

                    } catch (e) {
                        //Errors were being suppressed which was making debugging difficult.
                        console.error("Unable to render table: ", e.message);
                    }
                });

                return retVal;
            }
        };
        
        // Load the default menu template and give it a unique ID
        DellClarity.Resources.loadTemplate(returnTable.templateName);

        return returnTable;
    })();
}
