/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// A new version of this control will be created soon and will be compatible with the new API.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    DellClarity.SimpleGrid = (function () {

        var defaultOptions = {
            targetId: 'logsTable',
            height: 250,
            numberOfRows: 20,
            defaultSortOrder: DellClarity.Constants.DataSortingConstants.ASCENDING,
            enablePolling: false,
            enableSelect: true,
            multiselect: false,
            onRowSelect: null,
            onDataLoadComplete: null,
            pollingInterval: 60000,
            boundingContainer: null,
            autoResize: true
        };

        return {

            create: function (opts) {

                var options = $.extend(true, $.extend(true, {}, defaultOptions), opts);

                var defaultSortObject = new Object();
                if (options.defaultSortField) {
                    defaultSortObject.field = options.defaultSortField;
                    defaultSortObject.order = options.defaultSortOrder && (options.defaultSortOrder == DellClarity.Constants.DataSortingConstants.DESCENDING || options.defaultSortOrder == DellClarity.Constants.DataSortingConstants.ASCENDING)
                        ? options.defaultSortOrder
                        : DellClarity.Constants.DataSortingConstants.ASCENDING;
                } else {
                    defaultSortObject = null;
                }

                var retVal = {
                    timeout: null,
                    tableInitialized: false,
                    targetId: ko.observable(options.targetId),
                    url: ko.observable(options.url),
                    height: ko.observable(options.height != null ? options.height : 300),
                    pollingInterval: ko.observable(options.pollingInterval != null ? options.pollingInterval : 60000),
                    data: ko.observableArray(),
                    sortObj: ko.observableArray([defaultSortObject]),
                    paginationObj: ko.observable(DellClarity.ScrollableTable.getDefaultPaginationObject(0, options.numberOfRows)),
                    filterObj: ko.observableArray(),
                    allChecked: ko.observable(false),
                    selectedRowId: ko.observable(),
                    scrollbarPosition: ko.observable(),
                    multiselect: ko.observable(options.multiselect),
                    enableSelect: ko.observable(options.enableSelect),
                    boundingContainer: ko.observable(options.boundingContainer),
                    autoResize: ko.observable(options.autoResize),
                    loading: ko.observable(),

                    onRowSelect: function (id) {
                        if (!options.enableSelect) return;

                        if (options.onRowSelect) {
                            options.onRowSelect(id);
                        }
                    },
                    onDataLoadComplete: function (data) {
                        if (!options.onDataLoadComplete) return;

                        if (options.onDataLoadComplete) {
                            options.onDataLoadComplete(data);
                        }
                    },
                    refresh: function () {
                        var self = this;
                        self.loadData();
                    },

                    checkAll: function () {
                        var self = this;

                        var checked = self.allChecked();

                        self.allChecked(!checked);

                        ko.utils.arrayForEach(self.data(), function (row) {
                            row.checked(!checked);
                        });

                        return true;
                    },
                    selectRow: function (data) {
                        var self = retVal;
                        if (!self.enableSelect()) return;
                        self.selectedRowId(data.id);
                        self.onRowSelect(data.id);
                    },
                    updateSort: function (field) {
                        var self = this;

                        var currentSortObj = self.sortObj()[0];

                        if (currentSortObj.field == field) {
                            if (currentSortObj.order == DellClarity.Constants.DataSortingConstants.DESCENDING)
                                currentSortObj.order = DellClarity.Constants.DataSortingConstants.ASCENDING;
                            else currentSortObj.order = DellClarity.Constants.DataSortingConstants.DESCENDING;
                        } else {
                            currentSortObj.field = field;
                            currentSortObj.order = DellClarity.Constants.DataSortingConstants.ASCENDING;
                        }

                        self.refresh();
                    },
                    nextPage: function () {
                        var self = this;
                        var page = self.paginationObj().currentPage + 1;
                        if (page >= 0 && page <= retVal.lastPage() - 1) {
                            self.paginationObj().currentPage = page;
                            self.refresh();
                        }
                    },
                    previousPage: function () {
                        var self = this;
                        var page = self.paginationObj().currentPage - 1;
                        if (page >= 0 && page <= retVal.lastPage() - 1) {
                            self.paginationObj().currentPage = page;
                            self.refresh();
                        }
                    },
                    updatePage: function (page) {
                        var self = this;
                        if (page >= 1 && page <= retVal.lastPage()) {
                            self.paginationObj().currentPage = page - 1;
                            self.refresh();
                        }
                    },
                    rowSelected: function (data) {
                        if (data.checked() == true) return 'selected';
                        if (retVal.selectedRowId() && data.id == retVal.selectedRowId()) return 'selected';
                        return '';
                    },
                    loadData: function () {

                        var self = this;
                        self.loading(true);

                        if ($("#" + self.targetId()).length == 0) return;

                        var request = DellClarity.ScrollableTable.getBlankRequestObject();

                        request.criteriaObj.paginationObj = self.paginationObj();
                        request.criteriaObj.sortObj = self.sortObj();
                        request.criteriaObj.filterObj = self.filterObj();

                        var successHandler = function (response, data) {

                            var checkedRowsArray = self.checkedRows();
                            var selectedRowId = self.selectedRowId();

                            var selectedRowIdExists = false;

                            if (self.enableSelect()) {

                                ko.utils.arrayForEach(response, function (row) {

                                    if ($.inArray(row.id, checkedRowsArray) != -1) {
                                        row.checked = ko.observable(true);
                                    } else {
                                        row.checked = ko.observable(false);
                                        self.allChecked(false);
                                    }

                                    row.checked.subscribe(function (newValue) {
                                        if (newValue == false)
                                            self.allChecked(newValue);
                                    });

                                    // if this is our selected row update the observable
                                    if (selectedRowId && selectedRowId == row.id) {
                                        self.selectedRowId(row.id);
                                        selectedRowIdExists = true;
                                    }

                                });

                                if (!selectedRowIdExists) {
                                    if (response.length > 0) {
                                        self.selectedRowId(response[0].id);
                                        self.onRowSelect(response[0].id);
                                    } else {
                                        self.selectedRowId(null);
                                    }
                                } else {
                                    self.onRowSelect(self.selectedRowId());
                                }
                            }

                            self.onDataLoadComplete(response);

                            self.data(response);

                            if (data.criteriaObj) {
                                self.paginationObj(data.criteriaObj.paginationObj);
                                self.sortObj(data.criteriaObj.sortObj);
                                self.filterObj(data.criteriaObj.filterObj);
                            }

                            self.renderTable();
                            self.loading(false);

                            if (options.enablePolling) {
                                clearTimeout(self.timeout);
                                self.timeout = setTimeout(function () {
                                    self.loadData();
                                }, options.pollingInterval);
                            }
                        };

                        self.ajax = DellClarity.Ajax.GetRestfulJSON(self.url(), null, request, successHandler);
                    },
                    refreshTable: function () {
                        var self = this;
                        self.renderTable();
                    },
                    renderTable: function () {
                        var self = this;

                        var table = $("#" + self.targetId());

                        //save off scrollbar position before redrawing table
                        var currentTable = table.closest('.fht-tbody');
                        self.scrollbarPosition(currentTable.scrollTop());

                        table.fixedHeaderTable('destroy');

                        self.tableInitialized = false;

                        //table is empty so don't bother adding vertical scrollbar
                        if (self.data().length > 0) {

                            table.fixedHeaderTable(
                                {
                                    footer: true,
                                    cloneHeadToFoot: false,
                                    autoShow: true,
                                    height: self.height(),
                                    themeClass: 'dataTable',
                                    autoResize: false,
                                    create: function () {

                                        var currentHeight = table.height() - 25;

                                        var scrollablewrapper = table.closest('.fht-tbody');

                                        if (currentHeight < self.height() - 75) {
                                            scrollablewrapper.height(currentHeight);
                                        }

                                        if (!self.tableInitialized) {
                                            var element = $(table).closest('.fht-table-wrapper').find('.fht-thead thead');
                                            ko.cleanNode(element[0]);
                                            ko.applyBindings(self, element[0]);
                                        }

                                        //restore scroll position
                                        if (self.scrollbarPosition()) {
                                            scrollablewrapper.scrollTop(self.scrollbarPosition());
                                        }

                                        self.tableInitialized = true;
                                    }
                                });

                        }
                    }
                };

                retVal.checkedRows = ko.computed(function () {
                    var checkedRowArray = [];

                    if (retVal.enableSelect()) {

                        $.each(retVal.data(), function (i, row) {
                            if (row.checked()) checkedRowArray.push(row.id);
                        });

                    }
                    return checkedRowArray;
                });

                /* Pagination Helpers */
                retVal.itemCount = ko.computed(function () {
                    return retVal.paginationObj().totalItemsCount;
                });

                retVal.currentPage = ko.computed({
                    read: function () {
                        return retVal.paginationObj().currentPage + 1;
                    },
                    write: function (value) {
                        var page = value - 1;
                        if (page >= 0 && page < retVal.lastPage()) {
                            {
                                retVal.paginationObj().currentPage = page;
                                retVal.refresh();
                            }
                        } else {
                            retVal.paginationObj.valueHasMutated();
                        }
                    }
                });

                retVal.lastPage = ko.computed(function () {
                    return Math.ceil(retVal.paginationObj().totalItemsCount / retVal.paginationObj().rowCountPerPage);
                });

                retVal.firstDisplayedRowNumber = ko.computed(function () {
                    if (retVal.data().length == 0) return 0;
                    return (retVal.paginationObj().currentPage * retVal.paginationObj().rowCountPerPage) + 1;
                });

                retVal.lastDisplayedRowNumber = ko.computed(function () {
                    return (retVal.paginationObj().currentPage * retVal.paginationObj().rowCountPerPage) + retVal.data().length;
                });

                retVal.previousPageEnabled = ko.computed(function () {
                    return retVal.paginationObj().currentPage + 1 > 1;
                });

                retVal.nextPageEnabled = ko.computed(function () {
                    return retVal.paginationObj().currentPage + 1 < retVal.lastPage();
                });

                /* Helpers */

                retVal.selectedItemCount = ko.computed(function () {
                    var count = 0;
                    if (retVal.multiselect()) {

                        $.each(retVal.data(), function (i, row) {
                            if (row.checked()) count++;
                        });

                    } else {

                        if (!isNullOrEmpty(retVal.selectedRowId())) {
                            count = 1;
                        }

                    }
                    return count;
                });

                retVal.selectedRowData = ko.computed(function () {
                    var data = [];
                    if (retVal.multiselect()) {

                        $.each(retVal.data(), function (i, row) {
                            if (row.checked()) data.push(row);
                        });

                    } else {

                        if (!isNullOrEmpty(retVal.selectedRowId())) {
                            $.each(retVal.data(), function (i, row) {
                                if (row.id == retVal.selectedRowId()) data.push(row);
                            });
                        }

                    }
                    return data;
                });

                retVal.anyItemSelected = ko.computed(function () {
                    return retVal.selectedItemCount() > 0;
                });

                retVal.singleItemSelected = ko.computed(function () {
                    return retVal.selectedItemCount() == 1;
                });

                retVal.manyItemsSelected = ko.computed(function () {
                    return retVal.selectedItemCount() > 1;
                });

                $(window).bind('resizeTable', function () {
                    try {

                        if (options.autoResize) {
                            var container = null;

                            if (options.boundingContainer && $(options.boundingContainer).length > 0) {
                                container = $(options.boundingContainer);
                            } else if ($('#masterContainer').length > 0 || $('.masterContainer').length > 0) {
                                container = $('#masterContainer,.masterContainer');
                            } else if ($('#contentScrollBar').length > 0) {
                                container = $('#contentScrollBar');
                            }

                            //update height of table based on available height
                            if (container && container.length > 0) {
                                container.each(function (index, parentContainer) {
                                    var containerHeight = $(parentContainer).height();
                                    var tableElement = $(parentContainer).find(".fht-table-wrapper.standardTable");
                                    if (tableElement && tableElement.length > 0) {
                                        var topOfTable = tableElement.position().top;
                                        retVal.height(containerHeight - topOfTable);
                                    }
                                });
                            }
                        }

                        retVal.refreshTable();

                    } catch (e) {
                    }
                });

                return retVal;
            }
        };
    })();
}
