/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// A new version of this control will be created soon and will be compatible with the new API.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    //TODO: migrate to jQuery
    DellClarity.Tooltips = {

        tooltipDelayId: null,

        tooltipDelayTime: 500,

        delayShowTooltip: function (item) {

            if (item.length > 0) {
                // first, kill any other delayed tooltip timeouts
                if (tooltipDelayId) {
                    window.clearTimeout(tooltipDelayId);
                    tooltipDelayId = null;
                }

                // set timeout to delay showing tooltip
                tooltipDelayId = window.setTimeout(function () {
                    item.show();
                }, tooltipDelayTime);
            }

        },

        msgBoxWithTooltipOnMouseMove: function (event) {

            var containerOffset = $(event.currentTarget).offset();

            $(event.currentTarget)
                .find('.msgTooltip')
                .offset({ top: event.pageX - containerOffset.left + 140, left: event.pageY + containerOffset.top + 40 })
                .each(function () {
                    delayShowTooltip(this)
                });
        },

        hideTooltip: function (id) {

            if ($(id)) {
                // first, kill any delayed tooltip timeouts
                if (tooltipDelayId) {
                    window.clearTimeout(tooltipDelayId);
                    tooltipDelayId = null;
                }

                $(id).hide();
            }

        },

        tooltipMouseMoveHandler: function (event) {
            var targetElement = Event.element(event);
            var tooltip = $('msgTooltip');
            tooltip.down('.msgTooltipBottom').update(targetElement.tooltipText);

            var scrollOffsets = document.viewport.getScrollOffsets();

            var top = event.clientY - tooltip.getHeight();
            if (top < 0)
                top = scrollOffsets.top;
            else
                top += scrollOffsets.top;

            var screenWidth = document.viewport.getWidth();
            var tooltipWidth = tooltip.getWidth();
            var left = event.clientX + 4;
            if (left + tooltipWidth > screenWidth) {
                left = screenWidth - tooltipWidth + scrollOffsets.left;
                top = top - 4;
            } else
                left += scrollOffsets.left;

            tooltip.setStyle({
                left: left + 'px',
                top: top + 'px'
            });

            $('msgTooltip').show();
        },

        tooltipMouseOutHandler: function (event) {
            $('msgTooltip').hide();
        },

        addTooltipToElement: function (element) {

            var maxLength = element.contentMaxLength ? parseInt(element.contextMaxLength) : 25;

            element.stopObserving('mousemove', DellClarity.Tooltips.tooltipMouseMoveHandler);
            element.stopObserving('mouseout', DellClarity.Tooltips.tooltipMouseOutHandle);

            if (element.innerHTML && element.innerHTML.length > maxLength) {

                var currentText = element.innerHTML;

                if (currentText.length > maxLength) {
                    element.tooltipText = currentText + '';

                    currentText = currentText.substring(0, maxLength) + '...';
                    element.innerHTML = currentText;

                    if (!$('msgTooltip')) {
                        var tooltipDiv = new Element('div');
                        tooltipDiv.id = 'msgTooltip';
                        tooltipDiv.addClassName('msgTooltip');
                        tooltipDiv.hide();

                        $$('BODY')[0].appendChild(tooltipDiv);

                        var tmpDiv = new Element('div');
                        tmpDiv.addClassName('msgTooltipTop');
                        tooltipDiv.appendChild(tmpDiv);

                        var tmpDiv = new Element('div');
                        tmpDiv.addClassName('msgTooltipBottom');
                        tooltipDiv.appendChild(tmpDiv);
                    }

                    element.observe('mousemove', DellClarity.Tooltips.tooltipMouseMoveHandler);
                    element.observe('mouseout', DellClarity.Tooltips.tooltipMouseOutHandler);
                }
            }
        },

        addTooltips: function (containerId) {
            $$('#' + containerId + ' .autoEllipsis').each(function (targetElement) {
                DellClarity.Tooltips.addTooltipToElement(targetElement);
            });
        }
    };

    $(document).bind("viewReady.DellClarity", function (event, containerId) {
        setTimeout(function () {
            DellClarity.Tooltips.addTooltips(containerId);
        });
    });
}
