/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// A new version of this control will be created soon and will be compatible with the new API.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    if (!DellClarity.TopologyGraph) {
        DellClarity.TopologyGraph = (function () {

            var labelType, useGradients, nativeTextSupport, animate;

            (function () {
                var ua = navigator.userAgent,
                      iStuff = ua.match(/iPhone/i) || ua.match(/iPad/i),
                      typeOfCanvas = typeof HTMLCanvasElement,
                      nativeCanvasSupport = (typeOfCanvas == 'object' || typeOfCanvas == 'function'),
                      textSupport = nativeCanvasSupport
                        && (typeof document.createElement('canvas').getContext('2d').fillText == 'function');
                //I'm setting this based on the fact that ExCanvas provides text support for IE
                //and that as of today iPhone/iPad current text support is lame
                labelType = (!nativeCanvasSupport || (textSupport && !iStuff)) ? 'Native' : 'HTML';
                nativeTextSupport = labelType == 'Native';
                useGradients = nativeCanvasSupport;
                animate = !(iStuff || !nativeCanvasSupport);
            })();

            var selectLogger = {
                write: function (text) {
                    var elem = $("#log");
                    if (elem.length == 0)
                        return;
                    this.elem.innerHTML = text;
                    this.elem.style.left = (500 - this.elem.offsetWidth / 2) + 'px';
                }
            };

            var nodeOnClick = function (node) {
                var innerDetails = $("#inner-details");
                if (innerDetails.length == 0)
                    return;

                if (!node) return;
                // Build the right column relations list.  
                // This is done by traversing the clicked node connections.  
                var html = "<h4>" + node.name + "</h4><b> connections:</b><ul><li>",
                    list = [];
                node.eachAdjacency(function (adj) {
                    list.push(adj.nodeTo.name);
                });
                //append connections information  
                innerDetails.html(html + list.join("</li><li>") + "</li></ul>");
            };

            return {
                createModel: function () {
                    var retVal = {
                        currentZoomPercentage: 1.0,
                        graph: null,
                        options: {
                            Navigation: {
                                enable: true,
                                panning: 'avoid nodes',
                                zooming: false
                            },
                            Node: {
                                overridable: true,
                                color: "#F00",
                                height: 40,
                                width: 40
                            },
                            Edge: {
                                overridable: true,
                                color: '#23A4FF',
                                lineWidth: 0.4
                            },
                            Label: {
                                type: 'HTML', //Native or HTML  
                                size: 10,
                                style: 'bold',
                                color: "#000"
                            },
                            Tips: {
                                enable: true,
                                onShow: function (tip, node) {
                                    var count = 0;
                                    node.eachAdjacency(function () { count++; });
                                    tip.innerHTML = "<div class=\"tip-title\">" + node.name + "</div>"
                                        + "<div class=\"tip-text\"><b>connections:</b> " + count + "</div>";
                                }
                            },
                            Events: {
                                enable: true,
                                type: 'Native',
                                onMouseEnter: function () {
                                    retVal.graph.canvas.getElement().style.cursor = 'move';
                                },
                                onMouseLeave: function () {
                                    retVal.graph.canvas.getElement().style.cursor = '';
                                },
                                onDragMove: function (node, eventInfo, e) {
                                    var pos = eventInfo.getPos();
                                    node.pos.setc(pos.x, pos.y);
                                    retVal.graph.plot();
                                },
                                onTouchMove: function (node, eventInfo, e) {
                                    $jit.util.event.stop(e); //stop default touchmove event  
                                    this.onDragMove(node, eventInfo, e);
                                },
                                onClick: nodeOnClick
                            },
                            iterations: 200,
                            levelDistance: 130,
                            onCreateLabel: function (domElement, node) {
                                var style = domElement.style;
                                style.cursor = 'pointer';
                                style.textAlign = 'center';
                                $(domElement).click(function () {
                                    nodeOnClick(node);
                                });

                                $(domElement).html(node.name);
                            },
                            onPlaceLabel: function (domElement, node) {
                                var style = domElement.style;
                                var left = parseInt(style.left);
                                var top = parseInt(style.top);
                                var w = domElement.offsetWidth;
                                style.left = (left - w / 2) + 'px';
                                style.top = (top + 10) + 'px';
                                style.display = '';
                            }
                        },

                        setZoomPercentage: function (percentage) {
                            var currentPercentage = retVal.currentZoomPercentage;
                            var scaleToMatch = percentage / currentPercentage;
                            retVal.graph.canvas.scale(scaleToMatch, scaleToMatch);
                            retVal.currentZoomPercentage = percentage;
                        },

                        initializeGraph: function (containerId) {
                            var options = retVal.options;

                            var widthView = "800";
                            var heightView = "400";
                            var innerId = "topologyGraph";
                            $("#" + containerId).html("<div alt='' id='" + innerId + "' style='width:" + widthView + "px; height:" + heightView + "px;'></div>");
                            options.injectInto = innerId;
                            var fd = new $jit.ForceDirected(options);
                            retVal.graph = fd;
                        },

                        display: function (dataUrl, dataParameters) {
                            DellClarity.Ajax.PostJSON(dataUrl, dataParameters, function (listSpines) {
                                retVal.graph.loadJSON(listSpines);
                                retVal.graph.computeIncremental({
                                    iter: 20,
                                    property: 'end',
                                    onStep: function (perc) {
                                        selectLogger.write(perc + '% loaded...');
                                    },
                                    onComplete: function () {
                                        selectLogger.write('done');
                                        retVal.graph.animate({
                                            modes: ['linear'],
                                            //transition: $jit.Trans.Elastic.easeOut,
                                            duration: 1000
                                        });
                                    }
                                });
                            }, function () {
                                alert("there was a problem getting spines.");
                            });
                        }
                    };
                    return retVal;
                }
            };

        })();
    }
}
