/// <summary>
/// NOTE: This version of this control is not supported in DellClarity 2.5.
/// Use DellClarity.Controls.AdvancedWizard instead.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    DellClarity.Wizard = {
        wizardTemplateUrl: "templates/wizardtemplate.html",
        wizardTemplateName: "baseWizardTemplate",

        idIncrementer: 0,

        CreateWizard: function (wizardModel) {
            
            var id = DellClarity.Wizard.idIncrementer;
            DellClarity.Wizard.idIncrementer = DellClarity.Wizard.idIncrementer + 1;

            var finishAction = null;
            var cancelAction = null;
            var initialized = false;


            var resetSteps = function (startingIndex, wizard) {
                var steps = wizard.steps();
                for (var i = startingIndex; i < steps.length; i++) {
                    var step = steps[i];
                    step.resetModel(wizardModel);
                }
            };

            return {
                steps: ko.observableArray(),

                errors: ko.observableArray(),

                model: wizardModel,

                id: "wizard" + id,

                addStep: function (title, templateName, resetModel, isNextButtonVisibleTest, validationForNext) {
                    var self = this;
                    var newStep = {};

                    newStep.title = title;
                    newStep.validationForNext = validationForNext;
                    newStep.resetModel = resetModel;
                    newStep.index = self.steps().length;
                    newStep.templateName = templateName;
                    newStep.model = wizardModel;

                    newStep.isFirstStep = function () {
                        return newStep.index == 0;
                    };

                    newStep.isLastStep = function () {
                        return newStep.index == self.steps().length - 1;
                    };

                    newStep.isActive = ko.dependentObservable(function () {
                        return self.currentStepIndex() == newStep.index;
                    }, wizardModel);

                    self.steps.push(newStep);

                    newStep.arrowClass = ko.dependentObservable(function () {
                        var isLast = newStep.isLastStep();
                        var isActive = newStep.isActive();
                        var isSubsquentStepActive = isLast ? false : self.steps()[newStep.index + 1].isActive();

                        if (isActive) {
                            if (isLast)
                                return "arrow activeLast";
                            else
                                return "arrow activeToInactive";
                        } else {
                            if (isLast)
                                return "arrow inactiveLast";
                            else if (isSubsquentStepActive)
                                return "arrow inactiveToActive";
                            else
                                return "arrow inactiveToInactive";
                        }
                    }, wizardModel);

                    newStep.isNextButtonActive = isNextButtonVisibleTest ? ko.dependentObservable(isNextButtonVisibleTest, wizardModel) : ko.observable(true);


                    return newStep;
                },

                setFinishAction: function (theFinishAction) {
                    finishAction = theFinishAction;
                },

                setCancelAction: function (theCancelAction) {
                    cancelAction = theCancelAction;
                },

                currentStepIndex: ko.observable(),

                focus: function (inputName) {
                    if (inputName) {
                        $("#" + this.id + " .currentStepContainer input[name='" + inputName + "']").focus();
                    } else {
                        $("#" + this.id + " .currentStepContainer input").first().focus();
                    }
                },

                clearErrors: function () {
                    this.errors([]);
                },

                gotoStep: function (stepToGoto) {
                    if (stepToGoto >= this.currentStepIndex())
                        return;

                    this.clearErrors();

                    var actualStepToGoto = stepToGoto;
                    if (stepToGoto > this.currentStepIndex())
                        actualStepToGoto = this.currentStepIndex();

                    this.currentStepIndex(actualStepToGoto);
                    resetSteps(actualStepToGoto + 1, this);
                    this.focus();
                },

                nextStep: function () {
                    var step = this.steps()[this.currentStepIndex()];

                    var validationResult = null;

                    if (step.validationForNext)
                        validationResult = step.validationForNext();

                    if (validationResult != null) {
                        var errors = [];
                        if (typeof validationResult === "string")
                            errors.push(validationResult);
                        else
                            errors = validationResult;

                        if (errors.length > 0) {
                            this.errors(errors);
                            return;
                        }
                    }

                    this.clearErrors();

                    if (step.isLastStep()) {
                        finishAction();
                        return;
                    }

                    var newStepIndex = this.currentStepIndex() + 1;
                    this.currentStepIndex(newStepIndex);
                    this.focus();
                },

                cancel: function () {
                    cancelAction();
                },

                initialize: function () {
                    if (initialized) {
                        this.reset();
                    }

                    DellClarity.Resources.loadTemplate(DellClarity.Wizard.wizardTemplateName, DellClarity.Wizard.wizardTemplateUrl);

                    var self = this;

                    this.currentStep = ko.dependentObservable(function () {
                        return self.steps()[self.currentStepIndex()];
                    }, self);

                    initialized = true;
                    this.reset();
                },

                reset: function () {
                    this.clearErrors();
                    this.currentStepIndex(0);
                    resetSteps(0, this);
                }
            };
        }
    };
}
