/* **********************************************************
* This script highlights improper markup, deprecated CSS classes, 
* and other code problems so developers can easily spot them.
*
* This script is NOT needed in production.
********************************************************* */
DellClarity.Debug = (function () {
	/// <summary>
	/// Checks the DOM for markup errors, and applies the .codeError class to any errors that are found.
	/// </summary>
	function checkForCodeErrors() {
		// All DellClarity sites (semantic or not) should comply with these rules
		var codeErrors = [
			// These rules enforce accessible input forms
			{
				error: 'All <label>s must have a "for" attribute that indicates which field they apply to',
				selector: $('label:not([for])')
			},
			{
				error: 'This <label>\'s "for" attribute points to an id that doesn\'t exist',
				selector: $('label[for]').filter(function () { return document.getElementById($(this).attr('for')) == null; })
			},
			{
				error: '<label>s should be used to label input fields or other values, not just for blocks of text. Use a <p> instead.',
				selector: $('label[for]').not(function () { return $(document.getElementById($(this).attr('for'))).is(':input,.value,.fieldValue'); })
			},
			{
				error: 'All input elements (textboxes, checkboxes, drop-downs, etc.) must have an "id" attribute, for UI testing purposes, and so <label>s can point to them',
				selector: $(':input:not([id])')
			},
			{
				error: 'All input elements (textboxes, checkboxes, drop-downs, etc.) must have a <label>, for accessibility purposes.  Use the .visuallyhidden class to hide the <label> onscreen if needed.',
				selector: $('input[id]:not(.ipAddress+.ipAddress),textarea[id],select[id]').filter(function () { return $('label[for=' + this.id + ']').length == 0; })
			},
			{
				error: '<label>s should not contain input elements (textboxes, checkboxes, drop-downs, etc.). Move the input element outside of the <label> and use the "for" attribute instead.',
				selector: $('label:has(:input)')
			},
			{
				error: 'For accessibility reasons, All <label>s must contain text, even if the label is hidden onscreen using the .visuallyhidden class',
				selector: $('label').not(function () { return $.trim($(this).text()).length > 0; })
			},
			{
				error: 'All radio buttons must have a "name" attribute in addition to an "id"',
				selector: $('input[type=radio]:not([name])')
			},
			{
				error: '<select> elements do not have a "value" attribute',
				selector: $('select[value]')
			},
			{
				error: 'All <button>s must have a "type" attribute of "button", "reset", or "submit".',
				selector: $('button[type!=submit]').filter('[type!=button]').filter('[type!=reset]')
			}
		];

		// Semantic sites should also comply with these rules
		if (DellClarity.options.semanticMarkup) {
			codeErrors = codeErrors.concat([
				// These rules enforce proper use of semantic HTML5 tags
				{
					error: '<article>s should only be used for independent blocks of markup that have a <header>. Use a <div> or other semantically-appropriate element here instead.',
					selector: $('article:not(:has(header h1)):not([role=main],[role=dialog],[role=alertdialog])')
				},
				{
					error: '<section>s should only be used for independent blocks of markup that have a <header>. Use a <div> or other semantically-appropriate element here instead.',
					selector: $('section:not(:has(header h1))').not(function () { return $(this).parents('article[role=dialog]').length > 0; })
				},

				// These rules enforce proper use of DellClarity semantic CSS classes
				{
					error: 'This <label> should contain a <u> that wraps the text content of the label',
					selector: $('label:not(:has(u)), .label:not(:has(u))').not(function () { return $(this).parents('.label').length > 0; })
				},
				{
					error: 'This <u> isn\'t needed, because there\'s already one inside of the parent .label',
					selector: $('.label > u label > u')
				},
				{
					error: 'The contents of this <li> do not follow the correct pattern for .labelValuePairs. It should contain a label, then a value, optionally followed by related content such as a nested <ul>, <p>, <div>, etc.',
					selector: $('.labelValuePairs > li').not(function () {
						return $(this).children(
							'label:first-child+.value, ' +
							'.label:first-child+.value, ' +
							'input[type=checkbox].value:first-child+label, ' +
							'input[type=checkbox].value:first-child+.label, ' +
							'input[type=radio].value:first-child+label, ' +
							'input[type=radio].value:first-child+.label').length > 0;
					})
				},
				{
					error: 'Either use a <label> OR a <span class="label">, but not both',
					selector: $('label .label')
				},
				{
					error: 'The .label class should only be applied to the wrapper element, not its children',
					selector: $('.label .label')
				},
				{
					error: 'The .value class should be applied to the value, not the <label>',
					selector: $('label.value')
				},
				{
					error: 'The .value class should only be applied to the wrapper element, not its children',
					selector: $('.value .value')
				},
				{
					error: 'All <ul> and <ol> lists should have a CSS class applied. (e.g. .bulletedList, .inlineList, .labelValuePairs, etc.)',
					selector: $('ul:not([class])')
				},

				// These rules enforce proper use of semantic/accessible input forms
				{
					error: 'Input elements (textboxes, checkboxes, drop-downs, etc.) should be in a <fieldset>',
					selector: $('input,select,textarea').not(function () { return $(this).parents('fieldset').length > 0; })
				},
				{
					error: 'Input elements (textboxes, checkboxes, drop-downs, etc.) should be in a <fieldset> within a <form>',
					selector: $('input,select,textarea').not(function () { return $(this).parents('form').length > 0; })
				},
				{
					error: 'All <fieldset>s should contain a <legend>, even if the legend is not shown on-screen',
					selector: $('fieldset:not(:has(legend))')
				},
				{
					error: 'All <legend>s should contain text, for semantic and acceisibility purposes. Use the .visuallyHidden class if needed to hide the text on-screen',
					selector: $('fieldset > legend:empty')
				},

				// These rules enforce proper use of semantic/accessible tables/grids
				{
					error: 'If this table is displaying data (such as a list of objects, tabular information, etc.), then it should have the .dataTable class',
					selector: $('section table:not(.dataTable)')
				},
				{
					error: 'All <th> tags should have a scope attribute (col, row, etc.) for accessibility purposes',
					selector: $('th:not([scope])')
				},
				{
					error: 'All <tr> tags should be contained in a <thead>, <tbody>, or <tfoot>',
					selector: $('table > tr')
				},

				// These rules prohibit the use of deprectated stuff from previous (non-semantic) versions of DellClarity
				{
					error: 'This element has one or more deprecated CSS classes.',
					selector: $('section').find('.dataRow,.line,.dottedGrayBottomBorder,.standardTable,.padding10,.marginTop10')
				},
				{
					error: 'The .fieldValue  class has been deprectated.  Use the .value class instead.',
					selector: $('.fieldValue')
				},
				{
					error: 'The <span class="labelText"> syntax has been deprectated.  Use a <u> instead.',
					selector: $('.labelText')
				},
				{
					error: 'The useDatePicker binding is deprectated. Use <input type=\'date\'> or <input type=\'datetime\'',
					selector: $('[data-bind*=useDatePicker]')
				},
				{
					error: 'Do not bind events (e.g. click, submit, mouseover, keydown, etc.) in your HTML markup. Instead, bind your event handlers in the ViewModel using unobtrusive JavaScript practices.',
					selector: $('[data-bind*=click], [data-bind*=event], [data-bind*=submit], [onclick], [onmouseover], [onkeydown], [onkeypress]')
					// These exceptions are TEMPORARY, until the AdvancedWizard can be refactored to use unobtrusive JavaScript
						.not('#modalHelp, #modalClose, #idAdvanceWizardBackBtn, #idAdvancedWizardNextBtn, #idAdvancedWizardFinishBtn, #idAdvancedWizardCancelBtn')
						.not(function () { return $(this).parents('.advancedWizardSteps, .confirmationDialog').length > 0; })
				},
				{
					error: 'Do not use inline JavaScript for this link\'s "href" attribute.  Use a click event handler instead.',
					selector: $('[href^=javascript]')
					// These exceptions are TEMPORARY, until the AdvancedWizard can be refactored to use unobtrusive JavaScript
						.not(function () { return $(this).parents('.advancedWizardSteps').length > 0; })
				}
			]);
		}

		// Mark all of the previously-existing errors
		$('.codeError').data('alreadyHadError', true);

		// Highlight any elements that match any of the above error selectors
		for (var i = 0; i < codeErrors.length; i++) {
			var $matchedElements = codeErrors[i].selector;
			var errorMessage = codeErrors[i].error;

			$matchedElements.each(function (index, element) {
				var $element = $(element);

				// The .codeError class doesn't work on checkboxes and radio buttons, so select their parent instead
				if ($element.is('input[type=radio], input[type=checkbox]')) {
					$element = $element.parent();
				}

				$element
					.removeData('alreadyHadError')	// This element still has an error, so remove the pre-existing marker
					.not('.codeError')	// Don't re-flag elements that already have errors
					.addClass('codeError')	// Add a CSS class to visually indicate that there's an error
					.attr('title', errorMessage); // Add a tooltip that explains the error
			});
		}

		// Remove the .codeError class from any elements that no longer have errors
		var $errors = $('.codeError').filter(function () {
			var $element = $(this);
			if ($element.data('alreadyHadError')) {
				$element.removeData('alreadyHadError');
				$element.removeClass('codeError');
				$element.removeAttr('title');
				return false;
			}

			return true;
		});

		// Determine how many errors, if any, are on the page
		if ($errors.length > 0) {
			$('#codeErrorStatus').text($errors.length + ' markup errors');
			$('#codeErrorTools').removeClass('codeErrorTools_noErrors').addClass('codeErrorTools_errors');
		}
		else {
			$('#codeErrorStatus').text('No markup errors');
			$('#codeErrorTools').removeClass('codeErrorTools_errors').addClass('codeErrorTools_noErrors');
		}
	}

	/// <summary>
	/// Instantiates a MutationObserver object in a browser-independent way.
	/// </summary>
	function createMutationObserver(callback) {
		if (window.MutationObserver) {
			return new MutationObserver(callback);
		}
		else if (window.WebKitMutationObserver) {
			return new WebKitMutationObserver(callback);
		}
		else {
			// Return a faux MutationObserver implementation that just uses the above polling function
			console.warn('This browser does not support MutationObservers.  So DellClarity.Debug is using setInterval instead.');
			return {
				intervalId: 0,
				observe: function () { this.intervalId = setInterval(checkForCodeErrors, 3000); },
				disconnect: function () { clearInterval(this.intervalId); }
			};
		}
	}

	/// <summary>
	/// Monitors the DOM for changes, and runs a check for markup errors.
	/// </summary>
	var observer = createMutationObserver(function (mutations) {
		var self = this;

		// DOM changes happen VERY rapidly, so we don't want to call checkForCodeErrors on every change.
		var now = new Date().getTime();
		if (!self.lastCheckTime || (now - self.lastCheckTime > 500)) {
			// It has been a while since the last check, so check again
			self.lastCheckTime = now;
			console.log('DellClarity.Debug is checking for HTML markup errors');
			checkForCodeErrors();
		}

		// NOTE: DO NOT remove this line! It's here because it fixes a threading issue.
		// Basically, console.groupEnd() acts like a DoEvents call
		console.groupEnd();
	});

	return {
		/// <summary>
		/// Starts checking for markup errors whenever the DOM changes
		/// </summary>
		startErrorChecking: function () {
			if (!observer.isRunning) {
				observer.isRunning = true;
				observer.observe(document, { childList: true, subtree: true });
				$('#codeErrorToggle').text('Stop');
			}
		},

		/// <summary>
		/// Stops checking for markup errors whenever the DOM changes
		/// </summary>
		stopErrorChecking: function () {
			if (observer.isRunning) {
				observer.isRunning = false;
				observer.disconnect();
				$('.codeError').removeClass('codeError').removeAttr('title');
				$('#codeErrorStatus').text('No markup errors');
				$('#codeErrorTools').removeClass('codeErrorTools_errors').addClass('codeErrorTools_noErrors');
				$('#codeErrorToggle').text('Start');
			}
		},

		/// <summary>
		/// Starts or stops checking for markup errors whenever the DOM changes
		/// </summary>
		toggleErrorChecking: function () {
			if (observer.isRunning)
				DellClarity.Debug.stopErrorChecking();
			else
				DellClarity.Debug.startErrorChecking();
		},

		/// <summary>
		/// Checks the current DOM for markup errors, independent of whether auto-checking is currently running
		/// </summary>
		checkForCodeErrors: function () {
			checkForCodeErrors();
		},

		/// <summary>
		/// Shows or hides the error tools.
		/// </summary>
		toggleTools: function () {
			$('#codeErrorTools').toggleClass('codeErrorTools_minimized');
		}
	};

})();

$(document).ready(function () {
	// Start monitoring the document for DOM changes
	DellClarity.Debug.startErrorChecking();

	// Add helper buttons
	$('body').append(
		'<aside id="codeErrorTools" class="codeErrorTools_minimized codeErrorTools_noErrors">' +
			'<header><h1 id="codeErrorStatus">No markup errors</h1></header>' +
			'<button id="codeErrorToggle" type="button">Stop</button>' +
			'<button id="codeErrorManualCheck" type="button">Check Now</button>' +
		'</aside>');

	$('#codeErrorToggle').click(DellClarity.Debug.toggleErrorChecking);
	$('#codeErrorManualCheck').click(DellClarity.Debug.checkForCodeErrors);
	$('#codeErrorStatus').click(DellClarity.Debug.toggleTools);
});

