/// <summary>
/// NOTE: DellClarity.Helpers is not supported in DellClarity 2.5.
/// Most of its functions have been moved to other namespaces, or are no longer needed at all.
/// </summary>
if (DellClarity.Config.apiVersion < 2.5) {
    DellClarity.Helpers = {
        Validate: {
            ipAddress: function (ipAddress) {
                if (isNullOrEmpty(ipAddress)) return false;
                var ipaddressregex = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/;
                return ipaddressregex.test(ipAddress);
            },
            ipAddressWithOptionalPortOrHostname: function (sample) {
                if (isNullOrEmpty(sample)) return false;
                var ipaddressregex = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(:\d+)?$/;
                var hostnameregex = /^(?=.{1,255}$)[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?(?:\.[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?)*\.?$/;
                return ipaddressregex.test(sample) || hostnameregex.test(sample);
            }
        },
        getCookie: function (cookieName) {
            if (document.cookie.length > 0) {
                var cookieStart = document.cookie.indexOf(cookieName + "=");
                if (cookieStart != -1) {
                    cookieStart = cookieStart + cookieName.length + 1;
                    var cookieEnd = document.cookie.indexOf(";", cookieStart);
                    if (cookieEnd == -1) cookieEnd = document.cookie.length;
                    return unescape(document.cookie.substring(cookieStart, cookieEnd));
                }
            }
            return "";
        },
        setCookie: function (cookieName, value, expiredays) {
            var exdate = new Date();
            exdate.setDate(exdate.getDate() + expiredays);
            document.cookie = cookieName + "=" + escape(value) +
                ((expiredays == null) ? "" : "; expires=" + exdate.toGMTString());
        },
        translateYMDDateToJSON: function (ymd) {
            var formatedDate = ymd.gsub('-', '/') + ' 00:00:00';
            var utc = Date.parse(formatedDate);

            return DellClarity.Helpers.translateUTCDateToJSON(utc);
        },
        isDateString: function (value) {
            if (typeof value === 'string')
                return value.isDate();
            return false;
        },
        jsonDateToJsDate: function (json) {

            if (json && json.time) {
                return new Date(json.time);
            }
            return false;
        },
        formatJSONDateToString: function (obj) {

            var date = DellClarity.Helpers.jsonDateToJsDate(obj);
            if (false == date) {
                return '';
            }
            return date.getYmdHis();
        },
        formatDateString: function (str) {

            // assume str format:  "2010-10-10" or "2010-10-10 23:30:33" or "2010/10/10"

            var tmpDate;
            var dateParts;
            if (str.indexOf(' ') == -1) {

                dateParts = str.split('-');

                if (dateParts.size() == 3) {
                    tmpDate = new Date(dateParts[0], dateParts[1] - 1, dateParts[2]);

                } else {

                    dateParts = str.split('/');

                    tmpDate = new Date(dateParts[0], dateParts[1] - 1, dateParts[2]);

                }

            } else {
                var strParts = str.split(' ');

                dateParts = strParts[0].split('-');
                var timeParts = strParts[1].split(':');

                tmpDate = new Date(dateParts[0], dateParts[1] - 1, dateParts[2], timeParts[0], timeParts[1], timeParts[2]);
            }

            // "Mon, 01 Nov 2010 06:00:00 GMT"
            return tmpDate.toGMTString();

        },
        getValueFromStyle: function (str) {
            return parseInt(str.substring(0, (str.length - 2)));
        },
        postPageLoadUpdate: function () {
            //        // need to wait a second for everything to load
            //        window.setTimeout(function () {

            //            $(DellClarity.Config.mainContentContainerId).fire('MainContent:postPageLoad');

            //        }, 500);

        },
        concatPropertyValues: function (arrayOfItems, propertyName, joinSeparator, ignoreEmpty) {
            var names = [];
            for (var i = 0; i < arrayOfItems.length; i++) {
                var rawValue = arrayOfItems[i][propertyName];
                if (typeof rawValue === 'function')
                    rawValue = rawValue();

                if (ignoreEmpty != true || !isNullOrEmpty(rawValue))
                    names.push(rawValue);
            }
            return names.join(joinSeparator);
        },

        /**
        * given a list of ListItem-like objects, get the item's name that matches the given id
        * EX: 
        * list = [ { id: "1", name: "name1"}, { id: "2", name: "name2"} ]
        * listTwo = [ { id: "1", weirdName: "name1"}, { id: "2", weirdName: "name2"} ]
        * id = "2"
        *
        * getNameFromListItemById(id, list) ===> "name2"
        * getNameFromListItemById(id, listTwo, 'weirdName') ===> "name2"
        * if no match is found, null is returned.
        **/
        getNameFromListItemById: function (id, list, namePropertyName) {
            var propertyNameforName = namePropertyName ? namePropertyName : "name";

            var idValue = id;
            var listValue = list;
            if (typeof id === 'function')
                idValue = id();

            if (typeof list === 'function')
                listValue = list();

            for (var i = 0; i < listValue.length; i++) {
                var currentListItem = listValue[i];

                var listItemId = currentListItem.id;
                if (typeof listItemId === 'function')
                    listItemId = listItemId();

                if (listItemId == idValue)
                    return currentListItem[propertyNameforName];
            }
            return null;
        },


        /**
        * get a property from an object by giving a string representatin of the property
        * EX: 
        * startingObject = { 
        *   root: {
        *     subProp: "a property"
        *   }
        * }
        *
        * calling accessPropertyByString(startingObject, 'root.subProp') will return "a property"
        * if the property string doesn't resolve to something, null is returned
        **/
        accessPropertyByString: function (startingObject, stringRepresentationOfProperty) {
            return startingObject.getPropertyValue(stringRepresentationOfProperty);
        },
        ipaddressToLong: function (ipaddress) {
            if (isNullOrEmpty(ipaddress)) return 0;
            var d = ipaddress.split('.');
            return ((((((+d[0]) * 256) + (+d[1])) * 256) + (+d[2])) * 256) + (+d[3]);
        },
        moveUp: function (list, item) {
            var position = list.indexOf(item);
            if (position >= 0) {
                list.remove(item);
                list.splice(position - 1, 0, item);
            }
        },
        moveDown: function (list, item) {
            var position = list.indexOf(item);
            if (position >= 0) {
                list.remove(item);
                list.splice(position + 1, 0, item);
            }
        },

        getBlankRequestObject: function () {
            var request = new Object();
            request.requestObj = new Object();

            var criteria = new Object();
            request.criteriaObj = criteria;

            //Paging
            var paging = new Object();
            paging.currentPage = null;
            paging.rowCountPerPage = null;
            criteria.paginationObj = paging;

            //Sorting
            criteria.sortObj = [];

            //Filters
            criteria.filterObj = [];

            return request;
        },

        getDefaultPaginationObject: function (currentPage, numberOfRows) {
            //Paging
            var paging = new Object();
            paging.currentPage = currentPage;
            paging.rowCountPerPage = numberOfRows;
            paging.totalItemsCount = 0;
            return paging;
        },

        refreshTooltips: function () {
            $('.toolTip-Dynamic').livequery(function () {
                $(this).poshytip();
            });

            $('.toolTip-Dynamic-Sticky').livequery(function () {
                $(this).poshytip({
                    showOn: "manualClose"
                });
            });

        },

        /// <summary>
        /// Converts a standard JavaScript object (such as one returned from a JSON service call) to a Knockout.JS model object.
        /// All properties are converted to observables, and all arrays are converted to observable arrays.
        /// NOTE: In addition, this method converts any ISO 8601 date strings to JavaScript Date objects
        /// </summary>
        jsToKo: function (model, options, target) {
            var self = this;

            // Custom mapping logic
            // Note: This function calls itself recursively for arrays and objects
            if (!self.doCustomMapping) {
                self.doCustomMapping = function (obj) {
                    for (var prop in obj) {
                        var propValue = obj[prop];

                        if (typeof propValue === 'function') {
                            //do nothing
                        }
                        else if (propValue instanceof Array) {
                            // This is an array, so inspect each element to see if it's an object
                            for (var i = 0; i < propValue.length; i++) {
                                if (propValue instanceof Object) {
                                    // Performing custom mapping on this nested object
                                    self.doCustomMapping(propValue[i]);
                                }
                            }
                        }
                        else if (propValue instanceof Object) {
                            // Performing custom mapping on this nested object
                            self.doCustomMapping(propValue);
                        }
                        else if (DellClarity.Helpers.isDateString(propValue)) {
                            // This property is a date string, so convert it to a Date object
                            obj[prop] = moment(propValue).toDate();
                        }
                    }
                };
            }

            // Perform our custom mapping
            self.doCustomMapping(model);

            // Perform the standard KO mapping and return the result
            if (options && options != null)
                if (target && target != null)
                    return ko.mapping.fromJS(model, options, target);
                else
                    return ko.mapping.fromJS(model, options);
            else
                return ko.mapping.fromJS(model);
        },

        /// <summary>
        /// Resizes the global container elements according the browser window size
        /// </summary>
        resize: function() {
            try {
                var viewportHeight = $(window).height();
                var $mainContentContainer = $('#contentScrollBar');
                var $menuContainer = $('#menuScrollBar');

                // Only proceed if the container elements exist. Otherwise, the page is still loading and templates haven't been processed yet
                if ($mainContentContainer.length > 0) {
                    $mainContentContainer.height(viewportHeight - $mainContentContainer.offset().top - 60);
                }

                if ($menuContainer.length > 0) {
                    $menuContainer.height(viewportHeight - $menuContainer.offset().top - 35);
                }

                // Resize any top-level tabs
                DellClarity.Helpers.resizeTabs();

                // Resize the master/details area (without animation)
                // Note: This also calls resizeTabs, resizeGrid, and resizeTable
                DellClarity.Helpers.resizeMasterDetails(false);

            } catch(e) {
            }
        },

        /// <summary>
        /// Resizes the master/details area according to their container elements and the current detailsWindowState (minimized, normal, maximized)
        /// </summary>
        resizeMasterDetails: function (animate) {
            try {
                var $masterContainers = $('#masterContainer, .masterContainer');
                var $detailsContainers = $('#detailsContainer, .detailsContainer');

                var masterHeight, detailsHeight = 0;

                if ($masterContainers.length > 0 && $detailsContainers.length > 0) {
                    $masterContainers.each(function (index, masterContainer) {
                        var $contentContainer = $(masterContainer).parents('.tabsContainer,#contentScrollBar').first();
                        var detailsContainer = $detailsContainers[index];

                        // Enable/Disable animating the expand/collapse of the details panel
                        $contentContainer.toggleClass('animate', animate === true);

                        // Determine the state of the details panel
                        var detailsWindowState = DellClarity.WindowStates.Normal;
                        if ($.isArray(DellClarity.Navigation.currentViewModel.detailsWindowState()))
                            detailsWindowState = DellClarity.Navigation.currentViewModel.detailsWindowState()[index];
                        else if (DellClarity.Navigation.currentViewModel.detailsWindowState)
                            detailsWindowState = DellClarity.Navigation.currentViewModel.detailsWindowState();

                        // Resize the master/details panels
                        switch (detailsWindowState) {
                            case DellClarity.WindowStates.Minimized:
                                // Collapse the details panel so that only the header is visible
                                detailsHeight = $(detailsContainer).find('#detailsHeader').height() + 10;
                                $(detailsContainer).height(detailsHeight).addClass('collapsed');

                                // The master panel takes up whatever height is left
                                masterHeight = $contentContainer.height() - detailsHeight - 20;
                                $(masterContainer).removeClass('collapsed').height(masterHeight);
                                break;

                            case DellClarity.WindowStates.Maximized:
                                // Collapse the master panel
                                masterHeight = 0;
                                $(masterContainer).height(masterHeight).addClass('collapsed');

                                // The details panel takes up the full height
                                detailsHeight = $contentContainer.height();
                                $(detailsContainer).removeClass('collapsed').height(detailsHeight);

                                break;

                            default:
                                // Set the master/details panels to 50% of their container's height
                                masterHeight = Math.floor($contentContainer.height() / 2 - 10);
                                detailsHeight = masterHeight;
                                $(masterContainer).removeClass('collapsed').height(masterHeight);
                                $(detailsContainer).removeClass('collapsed').height(detailsHeight);
                        }


                        // If there are tabs in the details panel, then they need to be resized
                        if (animate) {
                            // Animations are still in progress, so we need to specify the final height
                            DellClarity.Helpers.resizeTabs($(detailsContainer).find('.tabsContainer')[0], detailsHeight);
                        }
                        else {
                            DellClarity.Helpers.resizeTabs();
                        }
                    });

                }

                // If there is a grid in the master panel, then it needs to be resized.
                // Wait for the animation to finish first
                window.setTimeout(function () {
                    $(window).trigger('resizeGrid');
                    $(window).trigger('resizeTable');
                }, 1000);

            }
            catch (e) {
            }
        },

        /// <summary>
        /// Resizes all tab controls on the page so that their content area fills their container.
        /// Also allows a specific tab control to be resized to a specific height.
        /// </summary>
        /// <param name="tabsElement">
        /// (Optional) A specific tab control that will be resized to the specified <paramref name="height"/>
        /// rather than being resized to the calculated height.  
        /// This is useful if the tab control's height cannot be calculated correctly, such as when it's
        /// in the process of a resizing animation.
        /// </param>
        /// <param name="parentHeight">
        /// (Optional) Specifies the height that the <paramref name="tabsElement"> tab control should be resized to.
        /// This is useful if the tab control's height cannot be calculated correctly, such as when it's
        /// in the process of a resizing animation.
        /// </param>
        resizeTabs: function (tabsElement, height) {
            $('.tabsContainer').each(function (index, element) {
                var $tabs = $(element);

                var calculatedHeight = 0;
                if (element == tabsElement) {
                    // Use the height param instead of calculating the height
                    calculatedHeight = height;
                }
                else {
                    // Calculate the height of the parent container
                    var $parent = $tabs.parents('#detailsContainer,.detailsContainer,#contentScrollBar');
                    if ($parent.length > 0)
                        calculatedHeight = $parent.outerHeight(true);
                }

                if (calculatedHeight > 0) {
                    var tabsBorderAndPadding = $tabs.outerHeight(true) - $tabs.height();
                    $tabs.height(calculatedHeight - $tabs.position().top - tabsBorderAndPadding);
                }
            });
        },

        /// <summary>
        /// Sets the window state (minimize, normal, maximized) of the details area of the current view model
        /// </summary>
        setDetailsWindowState: function (newState) {

            // Set the new window state
            //Check if there are more than one detailsWindowState in the currentViewModel (multiple tabs)
            if ($.isArray(DellClarity.Navigation.currentViewModel.detailsWindowState())) {
                var index = 0;
                if (DellClarity.Navigation.currentViewModel.activeTab())
                    index = DellClarity.Navigation.currentViewModel.activeTab() - 1;

                DellClarity.Navigation.currentViewModel.detailsWindowState()[index] = newState;
                //notify that the observable array item has been changed/updated
                DellClarity.Navigation.currentViewModel.detailsWindowState.valueHasMutated();
            } else {
                DellClarity.Navigation.currentViewModel.detailsWindowState(newState);
            }

            // Resize the master/details view accordingly
            DellClarity.Helpers.resizeMasterDetails(true);
        },

        /// <summary>
        /// Switches to the active tab of the details panel, and ensures that the tab contents are visible
        /// </summary>
        setActiveTab: function (tab) {
            // Get the tab number and the corresponding HTML element
            var tabNumber;
            var $tab;

            // The "tab" parameter can be the tab number or the HTML element
            if (isNumber(tab)) {
                tabNumber = tab;
                $tab = $('.tab')[tabNumber - 1];
            }
            else {
                // Determine the tab number by counting the number of preceeding siblings
                $tab = $(tab);
                tabNumber = $tab.prevAll('.tab').length + 1;
            }

            // Get the Knockout model object 
            var model = ko.dataFor($tab[0]);

            // Set the active tab
            if (model.activeTab) {
                model.activeTab(tabNumber);
            }
        },

        hideAlarmScoreboardDropDown: function (dropDown) {
            window.setTimeout(function () {
                $(dropDown).fadeOut('slow');
            }, 2500);
        },

        showAlarmScoreboardDropDown: function (clickedElement, dropDownElement) {
            $('.alertDropDown').hide();
            var offset = $(clickedElement).offset();
            var outerHeight = $(clickedElement).outerHeight(true);

            var windowWidth = $(window).width();

            var verticalOffset = 5;
            var horizontalOffset = 0;

            if ($.browser.msie) {
                verticalOffset = -10;
                horizontalOffset = -15;
            }

            $(dropDownElement).css({
                position: 'absolute',
                top: (offset.top + outerHeight) + verticalOffset + 'px',
                left: ((offset.left + $(dropDownElement).outerWidth(true)) >= windowWidth) ? (((offset.left + $(clickedElement).outerWidth()) - $(dropDownElement).outerWidth()) + horizontalOffset + 'px') : offset.left + horizontalOffset + 'px'
            }).show();
        },
        waitForFinalEvent: (function () {
            var timers = {};
            return function (callback, ms, uniqueId) {
                if (!uniqueId) {
                    uniqueId = "Don't call this twice without a uniqueId";
                }
                if (timers[uniqueId]) {
                    clearTimeout(timers[uniqueId]);
                }
                timers[uniqueId] = setTimeout(callback, ms);
            };
        })()
    };


    /**
    * Delegate - Simple code bind event handlers from different contexts.
    */
    DellClarity.Delegate = function (target, func) {
        return function () {
            func.apply(target, arguments);
        };
    };

    /**
    * Basic string templating support
    *
    * TODO: improve replacement performance using regular expressions. Try to remove the loop.
    */
    DellClarity.StringTemplate = function (template) {
        this.template = template;

        this.evaluate = function (parametersObject) {
            if (!parametersObject)
                return this.template;

            var tempResult = this.template;
            for (var parameterName in parametersObject) {
                var regExp = new RegExp('#\{' + parameterName + '\}', 'g');
                tempResult = tempResult.replace(regExp, parametersObject[parameterName]);
            }

            return tempResult;
        };
    };

    DellClarity.PollingToken = null;

    DellClarity.IdRequest = function (id) {
        this.id = id;
    };

    DellClarity.ShowPopupModal = function (title, text) {
        var modal = DellClarity.ConfirmationModal.CreateConfirmationModal(title, text, false, null, null);
        modal.show();
    };

    DellClarity.ClearErrors = function (mapper) {
        $.each(mapper, function (j, field) {

            if ($("#" + field.elementId))
                $("#" + field.elementId).removeClass("error");

            if ($("#" + field.tabId))
                $("#" + field.tabId).removeClass("error");

        });

    };

    DellClarity.ParseErrors = function (mapper, errorObj) {

        if (errorObj && errorObj.fldErrors) {
            $.each(errorObj.fldErrors, function (i, fldError) {

                var ErrorField = fldError.field;

                $.each(mapper, function (j, field) {

                    if (field.field == ErrorField) {
                        if ($("#" + field.elementId))
                            $("#" + field.elementId).addClass("error");

                        if ($("#" + field.tabId))
                            $("#" + field.tabId).addClass("error");
                    }

                });

            });
        }
    };

    DellClarity.Helpers.ToggleHelp = function (obj) {
        //go up to the header, then back down to the help details
        var helpdiv = $(obj);

        helpdiv.find('.infodev-help-details').toggle();
        helpdiv.find('.infodev-help-icon').toggleClass('expand_normal_12');
        helpdiv.find('.infodev-help-icon').toggleClass('collapse_normal_12');
    };

    DellClarity.Helpers.GetSelectedItemData = function (table) {

        var grid = $("#" + table);

        var selectedRowIds = grid.jqGrid('getGridParam', 'selarrrow');
        var selecteddata = [];

        $.each(selectedRowIds, function (idx, rowId) {
            var rowData = grid.jqGrid("getRowData", rowId);
            selecteddata.push(rowData);
        });

        return selecteddata;
    };

    DellClarity.Helpers.OpenWindow = function (url) {
        try {
            //	var winHelp = window.open(url, "_blank", 'status=1,scrollbars=1,toolbar=1,menubar=1,location=1');

            var width = 820;
            var height = 600;

            if (height > screen.height) {
                height = screen.height - 20;
            }

            var left = Math.round((screen.width - width) / 2);
            var top = Math.round((screen.height - height) / 2);

            var popup = window.open(url, 'popup', 'width=' + width + ',height=' + height + ',top=' + top + ',left=' + left + ',scrollbars=1,resizable=1,location=1,toolbar=1');
            if (popup.focus) {
                popup.focus();
            }
        } catch (e) {
        }
    };

    DellClarity.Helpers.htmlEscape = function (str) {
        return String(str)
            .replace(/&/g, '&amp;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;');
    };

    DellClarity.Helpers.autoScrollDiv = function (obj) {
        // setup animation of specified block "obj"
        var textHeight = obj.find(".autoscrollText").height();
        // calculate distance of animation
        var distance = parseInt(obj.css("top")) - (-textHeight);
        //duration will be distance / speed
        obj.animate(
            { top: -textHeight },  //scroll upwards
            50 * distance,
            "linear",
            function () {
                obj.css("top", 0); // scroll to start position
                DellClarity.Helpers.autoScrollDiv(obj);
            });
    };

    //Convert a array of strings to title case. 
    //Take a array of strings and return a version in title case where each word is capitalized.
    DellClarity.Helpers.arrayToTitleCase = function (array) {
        var str = array.toString().replace(',', ', ');
        return str.replace(/\w\S*/g, function (txt) { return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase(); });
    };

    DellClarity.Helpers.SetCulture = function (locale) {
        if (typeof Globalize != 'undefined')
            Globalize.culture(locale);

        if (locale && locale.length > 0) {
            var l = '';
            switch (locale) {
                case 'en-ca': l = 'en-ca'; break;
                case 'en-gb': l = 'en-gb'; break;
                case 'fr-ca': l = 'fr-ca'; break;
                case 'pt-br': l = 'pt-br'; break;
                case 'zn-cn': l = 'zn-cn'; break;
                case 'zn-tw': l = 'zn-tw'; break;
            }

            if (l == '') {
                switch (locale.split('-')[0]) {
                    case 'bg': l = 'bg'; break;
                    case 'ca': l = 'ca'; break;
                    case 'cv': l = 'cv'; break;
                    case 'da': l = 'da'; break;
                    case 'de': l = 'de'; break;
                    case 'es': l = 'es'; break;
                    case 'et': l = 'et'; break;
                    case 'eu': l = 'eu'; break;
                    case 'fi': l = 'fi'; break;
                    case 'fr': l = 'fr'; break;
                    case 'gl': l = 'gl'; break;
                    case 'hu': l = 'hu'; break;
                    case 'is': l = 'is'; break;
                    case 'it': l = 'it'; break;
                    case 'ja': l = 'ja'; break;
                    case 'ko': l = 'ko'; break;
                    case 'kr': l = 'kr'; break;
                    case 'nb': l = 'nb'; break;
                    case 'nl': l = 'nl'; break;
                    case 'pl': l = 'pl'; break;
                    case 'pt': l = 'pt'; break;
                    case 'ro': l = 'ro'; break;
                    case 'ru': l = 'ru'; break;
                    case 'sv': l = 'sv'; break;
                    case 'tr': l = 'tr'; break;
                }
            }

            if (l != '') {
                if (typeof moment != 'undefined')
                    moment.locale(l);
            } else {
                moment.locale('en');
            }
        }
    };

    DellClarity.Helpers.GlobalizeNumber = function (value, format) {
        return value.format(format);
    };

}
